#include <unistd.h>
#include "th02.hpp"
//-------------------------------------------------------------------------------------------------
namespace th02 {
	
	th02::th02() {
		i2c = new mraa::I2c(1);
		i2c->frequency(mraa::I2C_STD);
		i2c->address(0x40);
		recent_rh = 0;
		recent_temp = 0;
	}
	
	
	
	th02::~th02() { if ( i2c != 0 ) delete i2c; }
	
	
	
	void th02::check_sensor() {
		if ( read_id() != 0x50 ) throw(sensor_not_responding_exception());
	}
	
	
	
	void th02::heater_on() {
		unsigned char buffer = 0;
		// read config reg
		buffer = read_config();
		// set D1 of config reg
		buffer |= 0x02;
		// write config reg
		write_config(buffer);
	}
	
	
	
	void th02::heater_off() {
		unsigned char buffer = 0;
		// read config reg
		buffer = read_config();
		// reset D1 of config reg
		buffer &= 0xFD;
		// write config reg
		write_config(buffer);
	}
	
	
	
	float th02::get_rh(unsigned timeout_ms) {
		unsigned char buffer[2];
		unsigned timeout_counter = 0;
		// read config reg
		buffer[0] = read_config();
		// set D0 and reset D4 of config reg to start an RH conversion
		buffer[0] |= 0x01;
		buffer[0] &= 0xEF;
		// write config reg
		write_config(buffer[0]);
		// wait until rdy becomes 0
		while (1) {
			usleep(1);
			++timeout_counter;
			if ( timeout_counter == timeout_ms ) throw(conversion_timed_out_exception());
			if ( !read_rdy() ) break;
		}
		// read data reg
		read_data(buffer);
		recent_rh = (((((buffer[1] << 8) | buffer[0]) >> 4) >> 4) - 24);
		
		recent_temp = get_temp(timeout_ms);
		
		// linearization
		recent_rh = recent_rh - ( recent_rh*recent_rh * -0.00393 + recent_rh * 0.4008 - 4.7844 );

		// temperature compensation
		recent_rh = recent_rh + ( recent_temp - 30 ) * ( recent_rh * 0.00237 + 0.1973 );
		
		return recent_rh;
		
	}
	
	
	
	float th02::get_temp(unsigned timeout_ms) {
		unsigned char buffer[2];
		unsigned timeout_counter = 0;
		// read config reg
		buffer[0] = read_config();
		// set D0 and D4 of CONFIG to start a TEMPERATURE conversion
		buffer[0] |= 0x11;
		// write config reg
		write_config(buffer[0]);
		// wait until rdy becomes 0
		while (1) {
			usleep(1);
			++timeout_counter;
			if ( timeout_counter == timeout_ms ) throw(conversion_timed_out_exception());
			if ( !read_rdy() ) break;
		}
		// read data reg
		read_data(buffer);
		recent_temp = (((((buffer[1] << 8) | buffer[0]) >> 2) >> 5) - 50);
		return recent_temp;
	}
	
	
	
	unsigned char th02::read_config() {
		i2c->address(0x40);
		return i2c->readReg(0x03);
	}
	
	
	
	void th02::write_config(unsigned char data) {
		i2c->address(0x40);
		i2c->writeReg(0x03, data);
	}
	
	
	
	unsigned char th02::read_id() {
		i2c->address(0x40);
		return i2c->readReg(0x11);
	}
	
	
	
	unsigned char th02::read_rdy() {
		i2c->address(0x40);
		return i2c->readReg(0x00);
	}
	
	
	
	void th02::read_data(unsigned char output_buffer[2]) {
		i2c->address(0x40);
		output_buffer[0] = i2c->readReg(0x02);
		output_buffer[1] = i2c->readReg(0x01);
	}
	
}
//-------------------------------------------------------------------------------------------------
