#include <iostream>
#include "led_interface.hpp"
#include "mraa.hpp"
#include <unistd.h>
#include <time.h>
//-------------------------------------------------------------------------------------------------
namespace led_interface {

	unsigned char led_interface::dali_linearization_array[256] = {
			0,  77, 92,103,111,117,123,128,132,136,
			140,143,146,148,151,153,156,158,160,161,
			163,165,167,168,170,171,172,174,175,176,
			178,179,180,181,182,183,184,185,186,187,
			188,189,190,190,191,192,193,194,194,195,
			196,196,197,198,199,199,200,200,201,202,
			202,203,204,204,205,205,206,206,207,207,
			208,208,209,209,210,210,211,211,212,212,
			213,213,214,214,214,215,215,216,216,217,
			217,217,218,218,219,219,219,220,220,220,
			221,221,222,222,222,223,223,223,224,224,
			224,225,225,225,226,226,226,227,227,227,
			227,228,228,228,229,229,229,229,230,230,
			230,231,231,231,231,232,232,232,233,233,
			233,233,234,234,234,234,235,235,235,235,
			236,236,236,236,236,237,237,237,237,238,
			238,238,238,239,239,239,239,239,240,240,
			240,240,241,241,241,241,241,242,242,242,
			242,242,243,243,243,243,243,244,244,244,
			244,244,245,245,245,245,245,245,246,246,
			246,246,246,247,247,247,247,247,247,248,
			248,248,248,248,248,249,249,249,249,249,
			249,250,250,250,250,250,250,251,251,251,
			251,251,251,252,252,252,252,252,252,253,
			253,253,253,253,253,253,254,254,254,254,
			254,254,254,254,254,254};
	
	led_interface::led_interface() {
		mraa::Gpio* uart2_reset_pin;
		uart2_reset_pin = new mraa::Gpio(15);	// GPIO165
		uart2_reset_pin->dir(mraa::DIR_OUT);
		uart2_reset_pin->write(0);
		sleep(2);
		uart2_reset_pin->write(1);

		ppp = new ppp_link::ppp_link(true);
		for (int i = 0; i < 32; ++i) output_packet_buffer[i] = 0;
		output_packet_size = 0;
		for (int i = 0; i < 32; ++i) input_packet_buffer[i] = 0;
		input_packet_size = 0;
	}



	led_interface::~led_interface() { if ( ppp != 0 ) delete ppp; }



	void led_interface::check_connection() {
		
		// output packet buffer: 	0x09 0xFF cs 0x01 0xF0
		// required response:		0x19 0xFF cs 0x01 0xF0
		
		// set output packet buffer
		output_packet_buffer[0] = 0x09;
		output_packet_buffer[1] = 0xFF;
		output_packet_buffer[2] = 0x00;
		output_packet_buffer[3] = 0x01;
		output_packet_buffer[4] = 0xF0;
		
		// set output packet buffer size
		output_packet_size = 5;
		
		// set checksum
		set_checksum();
		
		// send the output packet
		ppp->transmit_a_packet(output_packet_size, (char*)output_packet_buffer);
		
		// wait for response
		input_packet_size = ppp->receive_a_packet(10000, (char*)input_packet_buffer);
		
		// check for checksum error response
		check_for_checksum_error_response();
		
		// check the response
		if (input_packet_size == 5 &&
			input_packet_buffer[0] == 0x19 &&
			input_packet_buffer[1] == 0xFF &&
			input_packet_buffer[3] == 0x01 &&
			input_packet_buffer[4] == 0xF0 &&
			check_checksum()) return;
		else throw(connection_error_exception());
		
	}



	void led_interface::set_luminance(unsigned char luminance) {
		
		unsigned char luminance_linearized = dali_linearization_array[luminance];

		// output packet buffer: 	0x09 0x00 cs 0x01 luminance
		// required response:		0x19 0xFF cs 0x01 0xF1
		
		// check luminance validity
		if ( luminance_linearized > 254 ) {
			throw(invalid_luminance_exception());
		}
		
		// set output packet buffer
		output_packet_buffer[0] = 0x09;
		output_packet_buffer[1] = 0x00;
		output_packet_buffer[2] = 0x00;
		output_packet_buffer[3] = 0x01;
		output_packet_buffer[4] = luminance_linearized;
		
		// set output packet buffer size
		output_packet_size = 5;
		
		// set checksum
		set_checksum();
		
		// send the output packet
		ppp->transmit_a_packet(output_packet_size, (char*)output_packet_buffer);
		
		// wait for response
		input_packet_size = ppp->receive_a_packet(10000, (char*)input_packet_buffer);
		
		// check for checksum error response
		check_for_checksum_error_response();
		
		// check the response
		check_command_acknowledge();
		
	}
	
	
	unsigned char led_interface::get_luminance() {

		// output packet buffer: 	0x01 address cs 0x01
		// required response:		0x19 address cs 0x01 luminance

		// set output packet buffer
		output_packet_buffer[0] = 0x01;
		output_packet_buffer[1] = 0x00;
		output_packet_buffer[2] = 0x00;
		output_packet_buffer[3] = 0x01;

		// set output packet buffer size
		output_packet_size = 4;

		// set checksum
		set_checksum();

		// send the output packet
		ppp->transmit_a_packet(output_packet_size, (char*)output_packet_buffer);

		// wait for response
		input_packet_size = ppp->receive_a_packet(10000, (char*)input_packet_buffer);

		// check for checksum error response
		check_for_checksum_error_response();

		// check the response
		if (input_packet_size == 5 &&
			input_packet_buffer[0] == 0x19 &&
			input_packet_buffer[1] == output_packet_buffer[1] &&
			input_packet_buffer[3] == 0x01 &&
			check_checksum()) {
				// response OK
		}
		else throw(invalid_response_exception());

		for ( unsigned i = 0; i < 256; ++i ) {
			if ( input_packet_buffer[4] == dali_linearization_array[i] ) return i;
		}
		return 0;

	}


	
	void led_interface::set_fade(unsigned char fade) {
		
		// output packet buffer: 	0x09 0x01 cs 0x01 fade
		// required response:		0x19 0xFF cs 0x01 0xF1
		
		// set output packet buffer
		output_packet_buffer[0] = 0x09;
		output_packet_buffer[1] = 0x01;
		output_packet_buffer[2] = 0x00;
		output_packet_buffer[3] = 0x01;
		output_packet_buffer[4] = fade;
		
		// set output packet buffer size
		output_packet_size = 5;
		
		// set checksum
		set_checksum();
		
		// send the output packet
		ppp->transmit_a_packet(output_packet_size, (char*)output_packet_buffer);
		
		// wait for response
		input_packet_size = ppp->receive_a_packet(10000, (char*)input_packet_buffer);
		
		// check for checksum error response
		check_for_checksum_error_response();
		
		// check the response
		check_command_acknowledge();
		
	}
	
	
	
	float led_interface::get_u(unsigned char uid) {
		
		// output packet buffer: 	0x01 address cs 0x02
		// required response:		0x19 address cs 0x02 UL UH
		
		// set output packet buffer
		output_packet_buffer[0] = 0x01;
		
		switch (uid) {
			case 1:
				output_packet_buffer[1] = 0x08;
				break;
			case 2:
				output_packet_buffer[1] = 0x0C;
				break;
			case 3:
				output_packet_buffer[1] = 0x10;
				break;
			case 4:
				output_packet_buffer[1] = 0x14;
				break;
			default: throw(invalid_channel_id_exception());
		}
		
		output_packet_buffer[2] = 0x00;
		output_packet_buffer[3] = 0x02;
		
		// set output packet buffer size
		output_packet_size = 4;
		
		// set checksum
		set_checksum();
		
		// send the output packet
		ppp->transmit_a_packet(output_packet_size, (char*)output_packet_buffer);
		
		// wait for response
		input_packet_size = ppp->receive_a_packet(10000, (char*)input_packet_buffer);
		
		// check for checksum error response
		check_for_checksum_error_response();
		
		// check the response
		if (input_packet_size == 6 &&
			input_packet_buffer[0] == 0x19 &&
			input_packet_buffer[1] == output_packet_buffer[1] &&
			input_packet_buffer[3] == 0x02 &&
			check_checksum()) {
				// response OK
		}
		else throw(invalid_response_exception());
		
		// calculate voltage
		return ((float)((input_packet_buffer[5] << 8) | input_packet_buffer[4]) / 10);
		
	}
	
	
	
	float led_interface::get_i(unsigned char iid) {
		
		// output packet buffer: 	0x01 address cs 0x02
		// required response:		0x19 address cs 0x02 IL IH
		
		// set output packet buffer
		output_packet_buffer[0] = 0x01;
		
		switch (iid) {
			case 1:
				output_packet_buffer[1] = 0x0A;
				break;
			case 2:
				output_packet_buffer[1] = 0x0E;
				break;
			case 3:
				output_packet_buffer[1] = 0x12;
				break;
			case 4:
				output_packet_buffer[1] = 0x16;
				break;
			default: throw(invalid_channel_id_exception());
		}
		
		output_packet_buffer[2] = 0x00;
		output_packet_buffer[3] = 0x02;
		
		// set output packet buffer size
		output_packet_size = 4;
		
		// set checksum
		set_checksum();
		
		// send the output packet
		ppp->transmit_a_packet(output_packet_size, (char*)output_packet_buffer);
		
		// wait for response
		input_packet_size = ppp->receive_a_packet(10000, (char*)input_packet_buffer);
		
		// check for checksum error response
		check_for_checksum_error_response();
		
		// check the response
		if (input_packet_size == 6 &&
			input_packet_buffer[0] == 0x19 &&
			input_packet_buffer[1] == output_packet_buffer[1] &&
			input_packet_buffer[3] == 0x02 &&
			check_checksum()) {
				// response OK
		}
		else {
			throw(invalid_response_exception());
		}
		
		// calculate current
		return 2*((float)((input_packet_buffer[5] << 8) | input_packet_buffer[4]));
		
	}



	void led_interface::set_checksum() {
		unsigned char checksum = 0;
		output_packet_buffer[2] = 0;
		for (int i = 0; i < output_packet_size; ++i) checksum += output_packet_buffer[i];
		output_packet_buffer[2] = checksum;
	}



	bool led_interface::check_checksum() {
		unsigned char checksum_sent = input_packet_buffer[2];
		unsigned char checksum_calculated = 0;
		input_packet_buffer[2] = 0;
		for (int i = 0; i < input_packet_size; ++i) checksum_calculated += input_packet_buffer[i];
		if ( checksum_sent != checksum_calculated ) throw(checksum_error_exception());
		return true;
	}
	
	
	
	void led_interface::check_command_acknowledge() {
		if (input_packet_size == 5 &&
			input_packet_buffer[0] == 0x19 &&
			input_packet_buffer[1] == 0xFF &&
			input_packet_buffer[3] == 0x01 &&
			input_packet_buffer[4] == 0xF1 &&
			check_checksum()) return;
		else throw(command_not_acknowledged_exception());
	}
	
	
	
	void led_interface::check_for_checksum_error_response() {
		if (input_packet_size == 5 &&
			input_packet_buffer[0] == 0x19 &&
			input_packet_buffer[1] == 0xFF &&
			input_packet_buffer[3] == 0x01 &&
			input_packet_buffer[4] == 0xF5 &&
			check_checksum()) throw(checksum_error_response_exception());
		else return;
	}
	
}
//-------------------------------------------------------------------------------------------------
