#include <iostream>
#include "ppp_link.hpp"
//-------------------------------------------------------------------------------------------------
namespace ppp_link {
	
	ppp_link::ppp_link(bool use_console_port) {
		if ( use_console_port == false ) uart = new mraa::Uart(0);
		else uart = new mraa::Uart("/dev/ttyMFD2");
		uart->setBaudRate(9600);
		uart->setMode(8, mraa::UART_PARITY_NONE, 1);
		uart->setFlowcontrol(false, false);
		
		for (int i = 0; i < 32; ++i) {
			encoded_receive_fifo[i] = 0;
			decoded_receive_fifo[i] = 0;
			encoded_transmit_fifo[i] = 0;
			decoded_transmit_fifo[i] = 0;
		}
		
		encoded_receive_fifo_pointer = 0;
		decoded_receive_fifo_pointer = 0;
		encoded_transmit_fifo_pointer = 0;
		decoded_transmit_fifo_pointer = 0;
		valid_frame_received = false;
	}



	ppp_link::~ppp_link() { if ( uart != NULL ) delete uart; }



	void ppp_link::reset() {
		for (int i = 0; i < 32; ++i) {
			encoded_receive_fifo[i] = 0;
			decoded_receive_fifo[i] = 0;
			encoded_transmit_fifo[i] = 0;
			decoded_transmit_fifo[i] = 0;
		}
		
		encoded_receive_fifo_pointer = 0;
		decoded_receive_fifo_pointer = 0;
		encoded_transmit_fifo_pointer = 0;
		decoded_transmit_fifo_pointer = 0;
		valid_frame_received = false;
	}



	unsigned char ppp_link::receive_a_packet(unsigned timeout, char packet_buffer[32]) {
		unsigned char number_of_decoded_bytes = 0;
		receive_a_frame(timeout);
		decode();
		number_of_decoded_bytes = decoded_receive_fifo_pointer;
		decoded_receive_fifo_pointer = 0;
		for (int i = 0; i < number_of_decoded_bytes; ++i) packet_buffer[i] = decoded_receive_fifo[i];
		return number_of_decoded_bytes;
	}



	void ppp_link::transmit_a_packet(unsigned char number_of_bytes, char packet_buffer[32]) {
		
		if ( number_of_bytes > 32 ) {
			reset();
			throw(transmit_buffer_overflow_exception());
		}
		
		decoded_transmit_fifo_pointer = number_of_bytes;
		encoded_transmit_fifo_pointer = 0;
		
		for (int i = 0; i < number_of_bytes; ++i) {
			decoded_transmit_fifo[i] = packet_buffer[i];
		}
		
		encode();
		
		uart->write(encoded_transmit_fifo, encoded_transmit_fifo_pointer);
		
	}



	//-------------------------------------------------------------------------------------------------
	//-------------------------------------------------------------------------------------------------
	//-------------------------------------------------------------------------------------------------



	void ppp_link::encode() {
		
		encoded_transmit_fifo[0] = 0x7E;
		++encoded_transmit_fifo_pointer;
		
		for (int i = 0; i < decoded_transmit_fifo_pointer; ++i) {
			if ( decoded_transmit_fifo[i] == 0x7D || decoded_transmit_fifo[i] == 0x7E ) {
				// insert ESC
				encoded_transmit_fifo[encoded_transmit_fifo_pointer] = 0x7D;
				++encoded_transmit_fifo_pointer;
				encoded_transmit_fifo[encoded_transmit_fifo_pointer] = decoded_transmit_fifo[i] ^ 0x20;
				++encoded_transmit_fifo_pointer;
			} else {
				encoded_transmit_fifo[encoded_transmit_fifo_pointer] = decoded_transmit_fifo[i];
				encoded_transmit_fifo_pointer++;
			}
		}
		
		encoded_transmit_fifo[encoded_transmit_fifo_pointer] = 0x7E;
		++encoded_transmit_fifo_pointer;
		decoded_transmit_fifo_pointer = 0;
		
	}



	void ppp_link::decode() {
		
		for ( int i = 1; i < (encoded_receive_fifo_pointer-1); ++i ) {
			if ( encoded_receive_fifo[i] != 0x7d ) {
				decoded_receive_fifo[decoded_receive_fifo_pointer] = encoded_receive_fifo[i];
				++decoded_receive_fifo_pointer;
			} else {
				// ESC character found
				++i;
				decoded_receive_fifo[decoded_receive_fifo_pointer] = encoded_receive_fifo[i] ^ 0x20;
				++decoded_receive_fifo_pointer;
			}
		}
		
		encoded_receive_fifo_pointer = 0;
		
	}



	void ppp_link::receive_a_frame(unsigned timeout) {
		
		typedef enum state_t { wait_for_flag, check_synchron, get_data } state_t;
		state_t state = wait_for_flag;
		char tmp = 0;
		encoded_receive_fifo_pointer = 0;
		valid_frame_received = false;
		
		while (1) {
			
			if ( uart->dataAvailable(timeout) ) {
				
				uart->read(&tmp, 1);
				
				if ( encoded_receive_fifo_pointer == 31 ) {
					// input fifo overflow
					// reception shall be terminated
					reset();
					throw(receive_fifo_overflow_exception());
				} else {
					
					// a byte has been received
					switch (state) {
						//-----------------------------------------------------------------------------
						case wait_for_flag:
							if ( tmp == 0x7e ) {
								// flag received
								encoded_receive_fifo[encoded_receive_fifo_pointer] = tmp;
								++encoded_receive_fifo_pointer;
								state = check_synchron;
							} else {
								// out of synchronization... skip byte, state unchanged
							}
							break;
						//-----------------------------------------------------------------------------
						case check_synchron:
							// check, whether the flag was a frame start or a frame end (because of a lost sync)
							if ( tmp != 0x7e ) {
								encoded_receive_fifo[encoded_receive_fifo_pointer] = tmp;
								++encoded_receive_fifo_pointer;
								state = get_data;
							} else {
								// the byte received after the first flag was another flag, which means that the second one is a frame start flag ->
								// now a function byte is expected -> state unchanged
								// the second flag is discarded -> the receive_fifo is unchanged, receive_fifo_pointer is unchanged
							}	
							break;
						//-----------------------------------------------------------------------------	
						case get_data:
							encoded_receive_fifo[encoded_receive_fifo_pointer] = tmp;
							++encoded_receive_fifo_pointer;
							
							// check whether it's the terminating flag
							if ( tmp == 0x7e ) {
								valid_frame_received = true;
								return;
							}
							
							break;
						//-----------------------------------------------------------------------------
					}
					
				}
				
			} else {
				reset();
				throw(uart_timeout_exception());
			}
				
		}
		
	}
	
}
//-------------------------------------------------------------------------------------------------
