#ifndef _PPP_LINK_
#define _PPP_LINK_
//-------------------------------------------------------------------------------------------------
#include "mraa.hpp"
#include <exception>
//-------------------------------------------------------------------------------------------------
namespace ppp_link {
	
	class ppp_link {
		
		public:
			/*
			 * If the use_console_port parameter is true, the constructor uses UART2 (/dev/ttyMFD2), otherwise it uses UART1.
			 */
			ppp_link(bool use_console_port);



			~ppp_link();



			/*
			 * The function resets the PPP link. All buffers and pointers are cleared.
			 */
			void reset();



			/*
			 * The function receives a PPP frame on the UART channel and decodes it. The decoded packet is stored in packet_buffer.
			 * In case of timeout, a uart_timeout_exception is thrown.
			 * If the received frame contains more than 32 bytes, a receive_fifo_overflow_exception is thrown.
			 * The functions returns the number of valid bytes stored in packet_buffer.
			 */
			unsigned char receive_a_packet(unsigned timeout, char packet_buffer[32]);



			/*
			 * The function encodes the message put into packet_buffer and transfers it through the UART channel.
			 * If number_of_bytes is greater than 32, a transmit_buffer_overflow_exception is thrown.
			 */
			void transmit_a_packet(unsigned char number_of_bytes, char packet_buffer[32]);
		
		private:
			mraa::Uart* uart;
			
			char encoded_receive_fifo[32];
			unsigned char encoded_receive_fifo_pointer;
			char decoded_receive_fifo[32];
			unsigned char decoded_receive_fifo_pointer;
			
			char encoded_transmit_fifo[64];
			unsigned char encoded_transmit_fifo_pointer;
			char decoded_transmit_fifo[32];
			unsigned char decoded_transmit_fifo_pointer;
			
			bool valid_frame_received;
			
			void encode();
			void decode();
			void receive_a_frame(unsigned timeout);
			
	};



	struct uart_timeout_exception : public std::exception {
		const char* what() const throw() { return "UART timed out!"; }
	};

	struct receive_fifo_overflow_exception : public std::exception {
		const char* what() const throw() { return "Receive FIFO overflow!"; }
	};

	struct transmit_buffer_overflow_exception : public std::exception {
		const char* what() const throw() { return "Transmit buffer overflow!"; }
	};
	
}
//-------------------------------------------------------------------------------------------------
#endif // _PPP_LINK_
//-------------------------------------------------------------------------------------------------
