#include <iostream>
#include <unistd.h>
#include <time.h>
#include "mraa.hpp"
#include "led_interface.hpp"
#include "th02.hpp"
//-------------------------------------------------------------------------------------------------
int led_interface_test () {

	unsigned iteration = 1;
	unsigned errors = 0;
	time_t time_stamp = 0;
	
	unsigned char dali_led_linearization_array[256] = {
		 52, 77, 92,103,111,117,123,128,132,136,140,143,146,148,151,153,156,158,160,161,163,165,167,168,170,171,172,174,175,176,178,179,
		180,181,182,183,184,185,186,187,188,189,190,190,191,192,193,194,194,195,196,196,197,198,199,199,200,200,201,202,202,203,204,204,
		205,205,206,206,207,207,208,208,209,209,210,210,211,211,212,212,213,213,214,214,214,215,215,216,216,217,217,217,218,218,219,219,
		219,220,220,220,221,221,222,222,222,223,223,223,224,224,224,225,225,225,226,226,226,227,227,227,227,228,228,228,229,229,229,229,
		230,230,230,231,231,231,231,232,232,232,233,233,233,233,234,234,234,234,235,235,235,235,236,236,236,236,236,237,237,237,237,238,
		238,238,238,239,239,239,239,239,240,240,240,240,241,241,241,241,241,242,242,242,242,242,243,243,243,243,243,244,244,244,244,244,
		245,245,245,245,245,245,246,246,246,246,246,247,247,247,247,247,247,248,248,248,248,248,248,249,249,249,249,249,249,250,250,250,
		250,250,250,250,250,250,250,250,250,250,250,250,250,250,250,250,250,250,250,250,250,250,250,250,250,250,250,250,250,250,250,250
	};
	
	unsigned char luminance_value = 5;
	//bool luminance_change_direction = false;
	
	mraa::Gpio* uart2_reset_pin;
	uart2_reset_pin = new mraa::Gpio(15);	// GPIO165
	uart2_reset_pin->dir(mraa::DIR_OUT);
	uart2_reset_pin->write(1);


	led_interface::led_interface ledif;
	th02::th02 trh;
	
	std::cout << "Connecting to Szasza's controller..." << std::endl;

	while (1) {
		try {
			ledif.check_connection();
			std::cout << "Connected to Szasza's controller" << std::endl;
			break;
		} catch (std::exception& e) {
			std::cout << e.what() << std::endl;
		}
		sleep(1);
	}
	
	std::cout << "Connecting to TH02 sensor..." << std::endl;
	try {
		trh.check_sensor();
		std::cout << "Connected to TH02 sensor" << std::endl;
	} catch (th02::sensor_not_responding_exception& e) {
		std::cout << e.what() << std::endl;
	}
	
	while (1) {
		
		time_stamp = time(NULL);
		std::cout << "Iteration #" << iteration << " @ " << ctime(&time_stamp);
		++iteration;
		std::cout << "Errors so far: " << errors << std::endl << std::endl;
		
		try {
			
			ledif.set_fade(0);

			//--------------------
			// adjust luminance --
			//--------------------
//			if ( luminance_value == 5 ) luminance_change_direction = false;
//			if ( luminance_value == 250 ) luminance_change_direction = true;
//
//			if ( !luminance_change_direction ) luminance_value += 5;
//			else luminance_value -= 5;

			if ( luminance_value == 5 ) luminance_value = 250;
			else luminance_value = 5;

			ledif.set_luminance(dali_led_linearization_array[luminance_value]);
			std::cout << "Luminance adjusted: " << ((double)(unsigned)luminance_value / 255) * 100 << " %" << std::endl;
			
			std::cout << "Adjusted luminance: " << (unsigned)ledif.get_luminance() << std::endl;

			sleep(3);

			//------------------------------------------
			// measure LED string voltage and current --
			//------------------------------------------
			std::cout << "LED string #1 voltage and current: ";
			std::cout << ledif.get_u(1) << " V; ";
			usleep(10e3);
			std::cout << ledif.get_i(1) << " mA; ";
			std::cout << std::endl;
			
			std::cout << "LED string #2 voltage and current: ";
			std::cout << ledif.get_u(2) << " V; ";
			usleep(10e3);
			std::cout << ledif.get_i(2) << " mA; ";
			std::cout << std::endl;

			std::cout << "LED string #3 voltage and current: ";
			std::cout << ledif.get_u(3) << " V; ";
			usleep(10e3);
			std::cout << ledif.get_i(3) << " mA; ";
			std::cout << std::endl;

			std::cout << "LED string #4 voltage and current: ";
			std::cout << ledif.get_u(4) << " V; ";
			usleep(10e3);
			std::cout << ledif.get_i(4) << " mA; ";
			std::cout << std::endl;

			//------------------------------------
			// measure temperature and humidity --
			//------------------------------------
			std::cout << "Temperature: " << trh.get_temp(1000) << " Celsius" << std::endl;
			std::cout << "Relative humidity: " << trh.get_rh(1000) << " %" << std::endl << std::endl;
			
			std::cout << "------------------------------------------------------" << std::endl << std::endl;
			
		} catch (std::exception& e) {
			++errors;
			std::cout << e.what() << std::endl;
		}
		
		sleep(3);
		
	}
	
}
//-------------------------------------------------------------------------------------------------
