#include <iostream>
#include <unistd.h>
#include <time.h>
#include <set>
#include "mraa.hpp"
#include "yitran_host.hpp"
#include "yitran_nc.hpp"
#include "yitran_node_descriptor.hpp"
//-------------------------------------------------------------------------------------------------
int application_nc () {

	//////////////////////////////////////
	// PLC network management resources //
	//////////////////////////////////////
	yitran_host::yitran_nc nc(false);
	unsigned network_size = 2;

	enum state_t {
		init,
		do_application_logic,
		check_for_login_messages_from_new_rss,
		request_login_confirmations
	};
	state_t state = init;

	yitran_host::runtime_indication recent_runtime_indication = yitran_host::unknown_indication;
	short new_rs_id = 0;
	std::set<short> logged_in_rss;
	unsigned char login_request = 0x7E;



	/////////////////////////////////
	// Application logic resources //
	/////////////////////////////////
	yitran_host::rs_hk_data_set recent_rs_hk_data_set;
	unsigned char luminance = 0;



	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



	while (1) {
		switch (state) {
			case init:
				std::cout << "Initialize NC..." << std::endl;
				nc.reset_and_go_online(network_size);
				std::cout << "NC initialized. Do application logic..." << std::endl;
				state = do_application_logic;
				break;

				//---------------------------------------------------------------------------------

			case do_application_logic:

				std::cout << std::endl << std::endl;

				// START APPLICATION LOGIC

				if ( luminance < 250 ) luminance += 50;
				else luminance = 0;

				for(std::set<short>::iterator p = logged_in_rss.begin(); p != logged_in_rss.end(); ++p) {
					try {

						// Send a data exchange request to the RS.
						nc.send_set_luminance_command((*p), luminance);
						recent_runtime_indication = nc.wait_for_runtime_indication(10000);
						if ( recent_runtime_indication == yitran_host::rx_intranetworking_packet ) {

							// Decode the RX intranetworking packet (fill user payload).
							nc.decode_rx_intranetworking_packet_indication();

							// Try to decode the RX intranetworking packet's payload as a data exchange reply.
							try {
								recent_rs_hk_data_set = nc.decode_data_exchange_reply();
								std::cout << "-------------------------------------------------------------------------------------------" << std::endl;
								std::cout << "RS #" << (*p) << " HK data set: " << std::endl;
								std::cout << "Pylon ID: " << recent_rs_hk_data_set.pylon_id << std::endl;
								std::cout << "LED string #1: " << recent_rs_hk_data_set.u1 << " V, " << recent_rs_hk_data_set.i1 << " mA" << std::endl;
								std::cout << "LED string #2: " << recent_rs_hk_data_set.u2 << " V, " << recent_rs_hk_data_set.i2 << " mA" << std::endl;
								std::cout << "LED string #3: " << recent_rs_hk_data_set.u3 << " V, " << recent_rs_hk_data_set.i3 << " mA" << std::endl;
								std::cout << "LED string #4: " << recent_rs_hk_data_set.u4 << " V, " << recent_rs_hk_data_set.i4 << " mA" << std::endl;
								std::cout << "Temperature: " << recent_rs_hk_data_set.temperature << " Celsius" << std::endl;
								std::cout << "Humidity: " << recent_rs_hk_data_set.humidity << "%" << std::endl;
								std::cout << "Luminance: " << 100 * recent_rs_hk_data_set.luminance / 255 << "%" << std::endl;
								std::cout << "Maintenance required: ";
								if ( recent_rs_hk_data_set.maintenance_required_flag == 0x01 ) std::cout << "YES" << std::endl;
								else std::cout << "NO" << std::endl;
								std::cout << "-------------------------------------------------------------------------------------------" << std::endl;
							} catch (std::exception& e) {
								std::cout << "RS #" << (*p) << " application logic failed: " << e.what() << std::endl;
								continue;
							}


						} else {
							std::cout << "It's not an RX intranetworking packet. Skip RS..." << std::endl;
							continue;
						}

					} catch (std::exception& e) {
						std::cout << "RS #" << (*p) << " application logic failed... continue..." << std::endl;
						continue;
					}
				}

				// END OF APPLICATION LOGIC

				std::cout << std::endl << std::endl;

				if ( logged_in_rss.size() < network_size ) {
					std::cout << "Waiting for new RSs..." << std::endl;
					state = check_for_login_messages_from_new_rss;
				} else {
					state = request_login_confirmations;
				}
				break;

				//---------------------------------------------------------------------------------

			case check_for_login_messages_from_new_rss:

				if ( logged_in_rss.size() == 0 ) {
					try {
						recent_runtime_indication = nc.wait_for_runtime_indication(60000);
					} catch (std::exception& e) {
						std::cout << "RSs seem to be silent. My modem is possibly fucked up... Better to re-initialize..." << std::endl;
						state = init;
						break;
					}
				} else {
					try {
						recent_runtime_indication = nc.wait_for_runtime_indication(5000);
					} catch (std::exception& e) {
						state = request_login_confirmations;
						break;
					}
				}

				if ( recent_runtime_indication == yitran_host::rx_intranetworking_packet ) {
					nc.decode_rx_intranetworking_packet_indication();
					if ( nc.recently_received_packet.user_payload_size == 3 && nc.recently_received_packet.user_payload[0] == 0x7E ) {
						new_rs_id = (nc.recently_received_packet.user_payload[2] << 8) | nc.recently_received_packet.user_payload[1];
						std::cout << "Login message received from " << new_rs_id << ". Inserting RS into logged_in_rss..." << std::endl;
						logged_in_rss.insert(new_rs_id);
						std::cout << "Sending approval to " << new_rs_id << std::endl;
						nc.acknowledged_packet_tx(new_rs_id, &login_request, 1, 5000);
					} else {
						// It's not a login message... skip...
					}
				} else {
					// Only RX Intranetworking Packets are handled.
				}

				break;

				//---------------------------------------------------------------------------------

			case request_login_confirmations:
				// Send login requests to the RSs already logged in and refresh the logged_in_rss set according to the responses received.
				// If a login message is received, which is not sent by an RS already logged in, the message is discarded (and processed in the next cycle
				// in the check_for_login_messages_from_new_rss state).

				for(std::set<short>::iterator p = logged_in_rss.begin(); p != logged_in_rss.end(); ++p) {

					std::cout << "Sending login request to " << (*p) << "..." << std::endl;

					try {
						nc.acknowledged_packet_tx((*p), &login_request, 1, 5000);
						// Wait for response
						recent_runtime_indication = nc.wait_for_runtime_indication(5000);
						if ( recent_runtime_indication == yitran_host::rx_intranetworking_packet ) {
							nc.decode_rx_intranetworking_packet_indication();
							if ( nc.recently_received_packet.user_payload_size == 3 && nc.recently_received_packet.user_payload[0] == 0x7E ) {
								std::cout << "Login message received." << std::endl;
								if ( ((nc.recently_received_packet.user_payload[2] << 8) | nc.recently_received_packet.user_payload[1]) == (*p) ) {
									// Login confirmation received from the RS.
									std::cout << "Login confirmed by RS. Send acknowledge..." << std::endl;
									// Send login approve.
									nc.acknowledged_packet_tx((*p), &login_request, 1, 5000);
									continue;
								} else {
									std::cout << "Login message received from a new RS. Retry to receive login message confirmation from the same RS." << std::endl;
									--p; // TODO!!!! Mi van, ha p = 0?
								}
							} else {
								// It's not a login message... skip...
							}
						} else {
							// Only RX Intranetworking PAckets are handled.
							std::cout << "Not an RX Intranetworking packet..." << std::endl;
						}
					} catch (std::exception& e) {
						std::cout << "Login request failed or confirmation timed out. Remove the RS from the logged_in_rss set." << std::endl;
						logged_in_rss.erase(p);
					}

				}

				// Print logged_in_rss
				std::cout << std::endl << "RSs logged in:" << std::endl;
				for(std::set<short>::iterator p = logged_in_rss.begin(); p != logged_in_rss.end(); ++p) {
					std::cout << (*p) << ", ";
				}
				std::cout << std::endl;

				sleep(5);

				// logged_in_rss set is refreshed. Go back to application logic
				state = do_application_logic;
				break;

				//---------------------------------------------------------------------------------

			//...

			default:
				//...
				break;
		}
	}

	return 0;

}
//-------------------------------------------------------------------------------------------------
