#include <iostream>
#include <unistd.h>
#include <time.h>
#include "mraa.hpp"
#include "yitran_host.hpp"
#include "yitran_nc.hpp"
#include "yitran_node_descriptor.hpp"
//-------------------------------------------------------------------------------------------------
int application_simplified_nc (const unsigned pylon_ids[2]) {

	yitran_host::yitran_nc nc(false);

	yitran_host::rs_hk_data_set recent_rs_hk_data_set;
	unsigned char luminance = 0;
	yitran_host::runtime_indication recent_runtime_indication = yitran_host::unknown_indication;

	unsigned char pylon_pointer = 0;

	std::cout << "Initialize NC..." << std::endl;
	nc.reset_and_go_online(2);



	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



	while (1) {

		if ( luminance == 0 ) luminance = 255;
		else luminance = 0;

		// Send data exchange request.
		try {
			nc.send_data_exchange_request(pylon_ids[pylon_pointer], luminance);
			std::cout << "Data exchange request sent to " << pylon_ids[pylon_pointer] << "." << std::endl;
		} catch (std::exception& e) {
			std::cout << e.what() << std::endl;
			if ( pylon_pointer == 1 ) pylon_pointer = 0;
			else ++pylon_pointer;
			continue;
		}

		//---------------------------------------------------------

		// Wait for the data exchange reply.
		while (1) {
			try {
				recent_runtime_indication = nc.wait_for_runtime_indication(10000);
				if ( recent_runtime_indication == yitran_host::rx_intranetworking_packet ) {
					nc.decode_rx_intranetworking_packet_indication();
					try {
						recent_rs_hk_data_set = nc.decode_data_exchange_reply();
					} catch (std::exception& e) {
						std::cout << e.what() << std::endl;
						break;
					}
					if ( recent_rs_hk_data_set.pylon_id == pylon_ids[pylon_pointer] ) {
						std::cout << "---------------------------------------" << std::endl;
						std::cout << "RS #" << pylon_ids[pylon_pointer] << " HK data set: " << std::endl;
						std::cout << "Pylon ID: " << recent_rs_hk_data_set.pylon_id << std::endl;
						std::cout << "LED string #1: " << recent_rs_hk_data_set.u1 << " V, " << recent_rs_hk_data_set.i1 << " mA" << std::endl;
						std::cout << "LED string #2: " << recent_rs_hk_data_set.u2 << " V, " << recent_rs_hk_data_set.i2 << " mA" << std::endl;
						std::cout << "LED string #3: " << recent_rs_hk_data_set.u3 << " V, " << recent_rs_hk_data_set.i3 << " mA" << std::endl;
						std::cout << "LED string #4: " << recent_rs_hk_data_set.u4 << " V, " << recent_rs_hk_data_set.i4 << " mA" << std::endl;
						std::cout << "Temperature: " << recent_rs_hk_data_set.temperature << " Celsius" << std::endl;
						std::cout << "Humidity: " << recent_rs_hk_data_set.humidity << "%" << std::endl;
						std::cout << "Luminance: " << 100 * recent_rs_hk_data_set.luminance / 255 << "%" << std::endl;
						std::cout << "---------------------------------------" << std::endl;
						break;
					} else continue;

				} else continue;

			} catch (std::exception& e) {
				std::cout << e.what() << std::endl;
				break;
			}
		}

		//---------------------------------------------------------

		sleep(1);
		if ( pylon_pointer == 1 ) {
			pylon_pointer = 0;
			if ( luminance == 0 ) luminance = 255;
			else luminance = 0;
		}
		else ++pylon_pointer;

	}

	return 0;

}
//-------------------------------------------------------------------------------------------------
