#include <iostream>
#include <unistd.h>
#include <time.h>
#include "mraa.hpp"
#include "led_interface.hpp"
#include "th02.hpp"
#include "yitran_rs.hpp"
//-------------------------------------------------------------------------------------------------
int application_simplified_rs (const unsigned pylon_id) {

	//////////////////////////////////////
	// PLC network management resources //
	//////////////////////////////////////
	yitran_host::yitran_rs rs(false);

	enum state_t {
		init,
		wait_for_low_level_nc_connection,
		wait_for_data_exchange_request
	};
	state_t state = init;

	yitran_host::runtime_indication recent_runtime_indication = yitran_host::unknown_indication;



	/////////////////////////////////
	// Application logic resources //
	/////////////////////////////////
	led_interface::led_interface ledif;
	th02::th02 trh;

	unsigned char measurement_data[256];
	for ( int i = 0; i < 256; ++i ) measurement_data[i] = 0;
	float tmp = 0;

	while (1) {
		std::cout << "Connecting to Szasza's controller..." << std::endl;
		try {
			ledif.check_connection();
			std::cout << "Connected to Szasza's controller" << std::endl;
			break;
		} catch (std::exception& e) {
			std::cout << e.what() << std::endl;
		}
		sleep(1);
	}

	while (1) {
		std::cout << "Connecting to TH02 sensor..." << std::endl;
		try {
			trh.check_sensor();
			std::cout << "Connected to TH02 sensor" << std::endl;
			break;
		} catch (std::exception& e) {
			std::cout << e.what() << std::endl;
		}
		sleep(1);
	}



	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



	while (1) {
		switch (state) {
			case init:
				std::cout << "Initialize RS..." << std::endl;
				rs.reset_and_go_online(2);
				std::cout << "RS initialized." << std::endl;
				std::cout << "Waiting for Connected to NC indication." << std::endl;
				state = wait_for_low_level_nc_connection;
				break;

				//-----------------------------------------------------------------------------------------------------------------------------------------------------------

			case wait_for_low_level_nc_connection:
				try {
					recent_runtime_indication = rs.wait_for_runtime_indication(30000);
					if ( recent_runtime_indication == yitran_host::connected_to_nc ) {
						std::cout << "Connected to NC indication received." << std::endl;
						std::cout << "Waiting for data exchange requests..." << std::endl;
						state = wait_for_data_exchange_request;
					}
				} catch (std::exception& e) {
					std::cout << "Connected to NC indication missing... Re-initialize and retry..." << std::endl;
					state = init;
				}
				break;

				//-----------------------------------------------------------------------------------------------------------------------------------------------------------

			case wait_for_data_exchange_request:

				try {
					recent_runtime_indication = rs.wait_for_runtime_indication(60000);
				} catch (std::exception& e) {
					std::cout << "NC is suspiciously silent. Re-initialize..." << std::endl;
					state = init;
					break;
				}

				rs.decode_rx_intranetworking_packet_indication();
				if ( 	rs.recently_received_packet.user_payload[0] == 0x80 &&
						rs.recently_received_packet.user_payload[1] == (pylon_id & 0xFF000000) >> 24 &&
						rs.recently_received_packet.user_payload[2] == (pylon_id & 0x00FF0000) >> 16 &&
						rs.recently_received_packet.user_payload[3] == (pylon_id & 0x0000FF00) >> 8 &&
						rs.recently_received_packet.user_payload[4] == (pylon_id & 0x000000FF) ) {

					// Data exchange request received with the RS's pylon ID.
					std::cout << "Data exchange request received." << std::endl;

					ledif.set_luminance(rs.recently_received_packet.user_payload[5]);
					std::cout << "Luminance adjusted: " << 100 * rs.recently_received_packet.user_payload[5] / 255 << "%" << std::endl;
					measurement_data[0] = 0x81;

					// Set pylon ID
					measurement_data[1] = (pylon_id & 0xFF000000) >> 24;
					measurement_data[2] = (pylon_id & 0x00FF0000) >> 16;
					measurement_data[3] = (pylon_id & 0x0000FF00) >> 8;
					measurement_data[4] = (pylon_id & 0x000000FF);

					// Collecting measurement data...
					// Reading LED string voltages
					tmp = ledif.get_u(1);
					measurement_data[5] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
					measurement_data[6] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
					measurement_data[7] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
					measurement_data[8] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));

					tmp = ledif.get_u(2);
					measurement_data[9] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
					measurement_data[10] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
					measurement_data[11] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
					measurement_data[12] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));

					tmp = ledif.get_u(3);
					measurement_data[13] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
					measurement_data[14] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
					measurement_data[15] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
					measurement_data[16] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));

					tmp = ledif.get_u(4);
					measurement_data[17] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
					measurement_data[18] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
					measurement_data[19] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
					measurement_data[20] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));

					// Reading LED string currents
					tmp = ledif.get_i(1);
					measurement_data[21] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
					measurement_data[22] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
					measurement_data[23] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
					measurement_data[24] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));

					tmp = ledif.get_i(2);
					measurement_data[25] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
					measurement_data[26] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
					measurement_data[27] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
					measurement_data[28] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));

					tmp = ledif.get_i(3);
					measurement_data[29] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
					measurement_data[30] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
					measurement_data[31] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
					measurement_data[32] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));

					tmp = ledif.get_i(4);
					measurement_data[33] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
					measurement_data[34] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
					measurement_data[35] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
					measurement_data[36] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));

					try {
						tmp = trh.get_temp(1000);
						measurement_data[37] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
						measurement_data[38] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
						measurement_data[39] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
						measurement_data[40] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));

						tmp = trh.get_rh(1000);
						measurement_data[41] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
						measurement_data[42] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
						measurement_data[43] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
						measurement_data[44] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));
					} catch (std::exception& e) {
						std::cout << e.what() << std::endl;
					}

					// Get the recent luminance
					measurement_data[45] = ledif.get_luminance();

					// Send data exchange reply back to the NC
					rs.acknowledged_packet_tx(1, measurement_data, 46, 2000);
					std::cout << "Data exchage response sent." << std::endl;
				}
				break;
		}
	}

	return 0;

}
//-------------------------------------------------------------------------------------------------
