#include <iostream>
#include <unistd.h>
#include <time.h>
#include "mraa.hpp"
#include "yitran_nc.hpp"
//-------------------------------------------------------------------------------------------------
int full_test_nc () {

	try {

		// A Yitran NC host is created using the non-console UART (UART1).
		yitran_host::yitran_nc nc(false);

		try {
			nc.initialize_and_establish_a_network(2, 5);
		} catch (std::exception& e) {
			std::cout << e.what() << std::endl;
			return 0;
		}

		printf("Connected RSs:\n");
		for ( unsigned i = 0; i < nc.number_of_connected_rss; ++i ) {
			printf("%d: %d\n", i, nc.rs_address_table[i]);
		}

		// Sending the data traffic enabled indication to the RS nodes.
		nc.send_data_traffic_enabled_indication();

		// Sending a luminance value to the RS node. The packet sent is a request too. As an answer to this packet,
		// the RS node sends the LED voltages, currents, temperature, and humidity values back as follows:
		// User payload byte		data
		//			0				LED string #1 voltage 	byte #3 [MSB]
		//			1				LED string #1 voltage 	byte #2
		//			2				LED string #1 voltage 	byte #1
		//			3				LED string #1 voltage 	byte #0 [LSB]
		//			4				LED string #2 voltage 	byte #3 [MSB]
		//			5				LED string #2 voltage 	byte #2
		//			6				LED string #2 voltage 	byte #1
		//			7				LED string #2 voltage 	byte #0 [LSB]
		//			8				LED string #3 voltage 	byte #3 [MSB]
		//			9				LED string #3 voltage 	byte #2
		//			10				LED string #3 voltage 	byte #1
		//			11				LED string #3 voltage 	byte #0 [LSB]
		//			12				LED string #4 voltage 	byte #3 [MSB]
		//			13				LED string #4 voltage 	byte #2
		//			14				LED string #4 voltage 	byte #1
		//			15				LED string #4 voltage 	byte #0 [LSB]
		//			16				LED string #1 current 	byte #3 [MSB]
		//			17				LED string #1 current 	byte #2
		//			18				LED string #1 current 	byte #1
		//			19				LED string #1 current 	byte #0 [LSB]
		//			20				LED string #2 current 	byte #3 [MSB]
		//			21				LED string #2 current 	byte #2
		//			22				LED string #2 current 	byte #1
		//			23				LED string #2 current 	byte #0 [LSB]
		//			24				LED string #3 current 	byte #3 [MSB]
		//			25				LED string #3 current 	byte #2
		//			26				LED string #3 current 	byte #1
		//			27				LED string #3 current 	byte #0 [LSB]
		//			28				LED string #4 current 	byte #3 [MSB]
		//			29				LED string #4 current 	byte #2
		//			30				LED string #4 current 	byte #1
		//			31				LED string #4 current 	byte #0 [LSB]
		//			32				Temperature				byte #3 [MSB]
		//			33				Temperature				byte #2
		//			34				Temperature				byte #1
		//			35				Temperature				byte #0 [LSB]
		//			36				RH						byte #3 [MSB]
		//			37				RH						byte #2
		//			38				RH						byte #1
		//			39				RH						byte #0 [LSB]
		yitran_host::runtime_indication recent_runtime_indication = yitran_host::unknown_indication;
		unsigned tmp = 0;
		unsigned char luminance_value = 20;
		bool luminance_change_direction = false;

		while (1) {

			// Determine new luminance value
			if ( luminance_value == 20 ) luminance_change_direction = false;
			if ( luminance_value == 220 ) luminance_change_direction = true;
			if ( !luminance_change_direction ) luminance_value += 200;
			else luminance_value -= 200;

			// Send the new luminance value to the RSs
			for ( unsigned i = 0; i < nc.number_of_connected_rss; ++i ) {

				// Send the new luminance value
				try {
					nc.acknowledged_packet_tx(nc.rs_address_table[i], &luminance_value, 1, 5000);
					//printf("Luminance adjust command sent to %d\n", nc.rs_address_table[i]);
				} catch (std::exception& e) {
					std::cout << e.what() << std::endl;
					continue;
				}

				// Wait for the response with the HK data
				try {
					recent_runtime_indication = nc.wait_for_runtime_indication(5000);
				} catch (std::exception& e) {
					std::cout << e.what() << std::endl;
					continue;
				}

				if ( recent_runtime_indication == yitran_host::rx_intranetworking_packet ) {
					nc.decode_rx_intranetworking_packet_indication();
					//printf("Packet received: Origin ID: %d, Final ID: %d, user payload size: %d\n", nc.recently_received_packet.origin_id, nc.recently_received_packet.final_id, nc.recently_received_packet.user_payload_size);
					//printf("User payload: ");
					//for ( unsigned i = 0; i < nc.recently_received_packet.user_payload_size; ++i ) printf("%d ", nc.recently_received_packet.user_payload[i]);
					//printf("\n");

					tmp |= (nc.recently_received_packet.user_payload[0] << 24);
					tmp |= (nc.recently_received_packet.user_payload[1] << 16);
					tmp |= (nc.recently_received_packet.user_payload[2] << 8);
					tmp |= nc.recently_received_packet.user_payload[3];
					printf("Node %d: LED string #1: %f V, ", nc.rs_address_table[i], *reinterpret_cast<float*>(&tmp));
					tmp = 0;
					tmp |= (nc.recently_received_packet.user_payload[16] << 24);
					tmp |= (nc.recently_received_packet.user_payload[17] << 16);
					tmp |= (nc.recently_received_packet.user_payload[18] << 8);
					tmp |= nc.recently_received_packet.user_payload[19];
					printf("%f mA\n", *reinterpret_cast<float*>(&tmp));
					tmp = 0;
					tmp |= (nc.recently_received_packet.user_payload[4] << 24);
					tmp |= (nc.recently_received_packet.user_payload[5] << 16);
					tmp |= (nc.recently_received_packet.user_payload[6] << 8);
					tmp |= nc.recently_received_packet.user_payload[7];
					//printf("LED string #2: %f V, ", *reinterpret_cast<float*>(&tmp));
					tmp = 0;
					tmp |= (nc.recently_received_packet.user_payload[20] << 24);
					tmp |= (nc.recently_received_packet.user_payload[21] << 16);
					tmp |= (nc.recently_received_packet.user_payload[22] << 8);
					tmp |= nc.recently_received_packet.user_payload[23];
					//printf("%f mA\n", *reinterpret_cast<float*>(&tmp));
					tmp = 0;
					tmp |= (nc.recently_received_packet.user_payload[8] << 24);
					tmp |= (nc.recently_received_packet.user_payload[9] << 16);
					tmp |= (nc.recently_received_packet.user_payload[10] << 8);
					tmp |= nc.recently_received_packet.user_payload[11];
					//printf("LED string #3: %f V, ", *reinterpret_cast<float*>(&tmp));
					tmp = 0;
					tmp |= (nc.recently_received_packet.user_payload[24] << 24);
					tmp |= (nc.recently_received_packet.user_payload[25] << 16);
					tmp |= (nc.recently_received_packet.user_payload[26] << 8);
					tmp |= nc.recently_received_packet.user_payload[27];
					//printf("%f mA\n", *reinterpret_cast<float*>(&tmp));
					tmp = 0;
					tmp |= (nc.recently_received_packet.user_payload[12] << 24);
					tmp |= (nc.recently_received_packet.user_payload[13] << 16);
					tmp |= (nc.recently_received_packet.user_payload[14] << 8);
					tmp |= nc.recently_received_packet.user_payload[15];
					//printf("LED string #4: %f V, ", *reinterpret_cast<float*>(&tmp));
					tmp = 0;
					tmp |= (nc.recently_received_packet.user_payload[28] << 24);
					tmp |= (nc.recently_received_packet.user_payload[29] << 16);
					tmp |= (nc.recently_received_packet.user_payload[30] << 8);
					tmp |= nc.recently_received_packet.user_payload[31];
					//printf("%f mA\n", *reinterpret_cast<float*>(&tmp));
					tmp = 0;
					tmp |= (nc.recently_received_packet.user_payload[32] << 24);
					tmp |= (nc.recently_received_packet.user_payload[33] << 16);
					tmp |= (nc.recently_received_packet.user_payload[34] << 8);
					tmp |= nc.recently_received_packet.user_payload[35];
					printf("Temperature: %f Celsius\n", *reinterpret_cast<float*>(&tmp));
					tmp = 0;
					tmp |= (nc.recently_received_packet.user_payload[36] << 24);
					tmp |= (nc.recently_received_packet.user_payload[37] << 16);
					tmp |= (nc.recently_received_packet.user_payload[38] << 8);
					tmp |= nc.recently_received_packet.user_payload[39];
					printf("RH: %f %%\n", *reinterpret_cast<float*>(&tmp));
					tmp = 0;

				} else continue;

			}

		}

		return 0;


	} catch (std::exception& e) {
		std::cout << e.what() << std::endl;
		return 0;
	}

	return 0;

}
//-------------------------------------------------------------------------------------------------
