#include <iostream>
#include <iomanip>
#include "mraa.hpp"
#include "led_interface.hpp"
#include "th02.hpp"
//-------------------------------------------------------------------------------------------------
float calculate_optimal_current(float ambient_temperature) {
	float ret_value = (0.00269381 * ambient_temperature * ambient_temperature) + (0.490951 * ambient_temperature) + 427.419;
	return ret_value;
}
//-------------------------------------------------------------------------------------------------
int temp_feedback() {

	unsigned char recent_luminance = 0;
	float recent_temperature = 0;
	float recent_humidity = 0;
	float recent_voltage = 0;
	float optimal_current = 0;
	float recent_current = 0;
	unsigned increase_request_counter = 0;
	unsigned decrease_request_counter = 0;

	led_interface::led_interface ledif;
	th02::th02 trh;

	while (1) {
		std::cout << "Connecting to Szasza's controller..." << std::endl;
		try {
			ledif.check_connection();
			std::cout << "Connected to Szasza's controller" << std::endl;
			break;
		} catch (std::exception& e) {
			std::cout << e.what() << std::endl;
		}
		sleep(1);
	}

	while (1) {
		std::cout << "Connecting to TH02 sensor..." << std::endl;
		try {
			trh.check_sensor();
			std::cout << "Connected to TH02 sensor" << std::endl;
			break;
		} catch (std::exception& e) {
			std::cout << e.what() << std::endl;
		}
		sleep(1);
	}

	// Acquire recent luminance
	while (1) {
		try {
			recent_luminance = ledif.get_luminance();
			break;
		} catch (...) {
			sleep(1);
		}
	}

	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	while (1) {

		// Acquire recent temperature
		while (1) {
			try {
				recent_temperature = trh.get_temp(1000);
				break;
			} catch (...) {
				sleep(1);
			}
		}

		// Acquire recent humidity
		while (1) {
			try {
				recent_humidity = trh.get_rh(1000);
				break;
			} catch (...) {
				sleep(1);
			}
		}

		// Calculate optimal forward current
		optimal_current = calculate_optimal_current(recent_temperature);

		// Acquire recent voltage
		while (1) {
			try {
				recent_voltage = ledif.get_u(2);
				break;
			} catch (...) {
				sleep(1);
			}
		}

		// Acquire recent current
		while (1) {
			try {
				recent_current = 0;
				for ( unsigned i = 0; i < 10; ++i )	recent_current += ledif.get_i(2);
				recent_current = recent_current / 10;
				break;
			} catch (...) {
				sleep(1);
			}
		}

		// Send new luminance value
		if ( recent_current < optimal_current ) {
			decrease_request_counter = 0;
			++increase_request_counter;
			if ( recent_luminance <= 254 && increase_request_counter > 10 ) {
				increase_request_counter = 0;
				++recent_luminance;
				while(1) {
					try {
						ledif.set_luminance(recent_luminance);
						break;
					} catch (...) {
						sleep(1);
					}
				}
			}
		}
		else if ( recent_current > optimal_current ) {
			increase_request_counter = 0;
			++decrease_request_counter;
			if ( recent_luminance >= 1 && decrease_request_counter > 10 ) {
				decrease_request_counter = 0;
				--recent_luminance;
				while (1) {
					try {
						ledif.set_luminance(recent_luminance);
						break;
					} catch (...) {
						sleep(1);
					}
				}
			}
		}

		std::cout << std::setw(3) << recent_temperature << " C,  " << std::setw(8) << recent_humidity << " %,  " << std::setw(4) << recent_voltage << " V,   " << std::setw(3) << (unsigned)recent_current << " mA, (optimal: " << std::setw(3) << (unsigned)optimal_current << " mA),  " << "Luminance: " << (unsigned)recent_luminance << std::endl;
		//usleep(500e3);

	}

	return 0;

}
//-------------------------------------------------------------------------------------------------
