"use strict";

const SmartSSL = require('./build/Release/SmartSSL');

try {
    let nc = new SmartSSL.YitranNc(false);
    let network_size = 2;

    const state_t = {
        init: 0,
        do_application_logic: 1,
        check_for_login_messages_from_new_rss: 2,
        request_login_confirmations: 3
    };

    const runtime_indication = {
            connectivity_status_with_rs: 0,
            new_connection_to_nc: 1,
            disconnected_from_nc: 2,
            connected_to_nc: 3,
            rx_intranetworking_packet: 4,
            unknown_indication: 5
    };

    let state = state_t.init;
    let recent_runtime_indication = runtime_indication.unknown_indication;
    let new_rs_id = 0;

    let logged_in_rss = [];
    const login_request = [0x7E];

	/////////////////////////////////
	// Application logic resources //
	/////////////////////////////////
	let recent_rs_hk_data_set;
	let luminance = 0;
		
	function calculate_optimal_forward_current(temperature) {
		//float ret_value = (0.003881554 * ambient_temperature * ambient_temperature) + (0.626921775 * ambient_temperature) + 477.6894677;
		//float ret_value = (0.003881554 * ambient_temperature * ambient_temperature) + (0.626921775 * ambient_temperature) + 220;
		let ret_value = 20;
		return ret_value;
	}
    
    function application_logic()
	{
		console.log("\n");
		// START APPLICATION LOGIC
		if ( luminance < 250 ) luminance += 50;
		else luminance = 0;
		
		for (let i = 0; i < logged_in_rss.length; ++i) {
			let p = logged_in_rss[i];
			try {
				// Send a data exchange request to the RS.
				nc.send_set_luminance_command(p, luminance);
				recent_runtime_indication = nc.wait_for_runtime_indication(10000);
				if ( recent_runtime_indication == runtime_indication.rx_intranetworking_packet ) {
					// Decode the RX intranetworking packet (fill user payload).
					nc.decode_rx_intranetworking_packet_indication();
					
					// Try to decode the RX intranetworking packet's payload as a data exchange reply.
					try {
						recent_rs_hk_data_set = nc.decode_data_exchange_reply();
						console.log("-------------------------------------------------------------------------------------------");
						console.log("RS #",p," HK data set: ");
						console.log(recent_rs_hk_data_set);
						if ( recent_rs_hk_data_set.maintenance_required_flag == 0x01 )
							console.log("YES");
						else console.log("NO");
						console.log("-------------------------------------------------------------------------------------------");
					} catch (e) {
						console.log("RS #",p," application logic failed: ", e);
						continue;
					}
				} else {
					console.log("It's not an RX intranetworking packet. Skip RS...");
					continue;
				}

			} catch (e) {
				console.log("RS #" << p << " application logic failed... continue...");
				continue;
			}
		}				
		// END OF APPLICATION LOGIC			
		console.log("\n");
	}
	
    while(true) {
		switch(state) {
			case state_t.init: 
				console.log("Initialize NC...");
				nc.reset_and_go_online(network_size);
				console.log("NC initialized. Do application logic...");
				state = state_t.do_application_logic;
				break;
			
			case state_t.do_application_logic:
				application_logic();
				
				if (logged_in_rss.length < network_size ) {
					console.log("Waiting for new RSs...");
					state = state_t.check_for_login_messages_from_new_rss;
				} else {
					state = state_t.request_login_confirmations;
				}
				break;
			
			case state_t.check_for_login_messages_from_new_rss:
				if ( logged_in_rss.length == 0 ) {
					try {
						recent_runtime_indication = nc.wait_for_runtime_indication(60000);
					} catch (e) {
						console.log("RSs seem to be silent. My modem is possibly fucked up... Better to re-initialize...");
						state = state_t.init;
						break;
					}
				} else {
					try {
						recent_runtime_indication = nc.wait_for_runtime_indication(5000);
					} catch (e) {
						state = state_t.request_login_confirmations;
						break;
					}
				}
				
				if ( recent_runtime_indication == runtime_indication.rx_intranetworking_packet ) {
					nc.decode_rx_intranetworking_packet_indication();
					if ( nc.recently_received_packet.user_payload_size == 3 && nc.recently_received_packet.user_payload[0] == 0x7E ) {
						new_rs_id = (nc.recently_received_packet.user_payload[2] << 8) | nc.recently_received_packet.user_payload[1];
						console.log("Login message received from " + new_rs_id + ". Inserting RS into logged_in_rss...");
						logged_in_rss.push(new_rs_id);
						console.log("Sending approval to " + new_rs_id);
						nc.acknowledged_packet_tx(new_rs_id, login_request, 5000);
					} else {
						// It's not a login message... skip...
					}
				} else {
					// Only RX Intranetworking Packets are handled.
				}

				break;
				
			case state_t.request_login_confirmations:
				// Send login requests to the RSs already logged in and refresh the logged_in_rss set according to the responses received.
				// If a login message is received, which is not sent by an RS already logged in, the message is discarded (and processed in the next cycle
				// in the check_for_login_messages_from_new_rss state).
				
				for (let i = 0; i < logged_in_rss.length; ++i) {
					let p = logged_in_rss[i];
					console.log("Sending login request to " + p +"...");

					try {
						nc.acknowledged_packet_tx(p, login_request, 5000);
						
						// Értékadás hiányzott!!!!
						recent_runtime_indication = nc.wait_for_runtime_indication(5000);
						
						// A feltétel nem tudott teljesülni, mert a packetTX nem várta meg az ack reply packet-eket.
						// Ezek itt estek be, de ezek meg nem intranetwörking packet típusúak... 
						if ( recent_runtime_indication == runtime_indication.rx_intranetworking_packet ) {
							nc.decode_rx_intranetworking_packet_indication();
							if ( nc.recently_received_packet.user_payload_size == 3 && nc.recently_received_packet.user_payload[0] == 0x7E ) {
								console.log("Login message received.");
								if ( ((nc.recently_received_packet.user_payload[2] << 8) | nc.recently_received_packet.user_payload[1]) == p ) {
									// Login confirmation received from the RS.
									console.log("Login confirmed by RS. Send acknowledge...");
									// Send login approve.
									nc.acknowledged_packet_tx(p, login_request, 5000);
									continue;
								} else {
									console.log("Login message received from a new RS. Retry to receive login message confirmation from the same RS.");
									--i; // TODO!!!! Mi van, ha i = 0?
								}
							} else {
								// It's not a login message... skip...
								console.log("Not a login message");
							}
						} else {
							// Only RX Intranetworking PAckets are handled.
							console.log("Not an RX Intranetworking packet");
						}
					} catch (e) {
						console.log("Login request failed or confirmation timed out. Remove the RS from the logged_in_rss set.");
						
						// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
						//logged_in_rss = []; // Itt nem kell kitörölni az egész set-et, csak a p-t eltávolítani!!!
						let index = logged_in_rss.indexOf(p);
						logged_in_rss.splice(index, 1);
						// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
					}

				}
				
			// Missing stuff: application_nc.cpp: line 203-213
			console.log("RSs logged in: ");
			for (let i = 0; i < logged_in_rss.length; ++i) {
				let p = logged_in_rss[i];
				console.log(p + ", ");
			}
			state = state_t.do_application_logic;
								
			break;
			
		}
		
	}

} catch (e) {
    console.error(e);
    process.exit(1);
}
