#include <node.h>
#include "node_wrapper_th02.hpp"

namespace node_wrappers {

using v8::Function;
using v8::FunctionCallbackInfo;
using v8::FunctionTemplate;
using v8::Isolate;
using v8::Local;
using v8::Number;
using v8::Object;
using v8::Persistent;
using v8::String;
using v8::Value;
using v8::Exception;

Persistent<Function> TH02::constructor;


void TH02::Init(Local<Object> exports) {
    Isolate* isolate = exports->GetIsolate();

    // Prepare constructor template
    Local<FunctionTemplate> tpl = FunctionTemplate::New(isolate, New);
    tpl->SetClassName(String::NewFromUtf8(isolate, "TH02"));
    tpl->InstanceTemplate()->SetInternalFieldCount(1);

    // Prototype
    NODE_SET_PROTOTYPE_METHOD(tpl, "check_sensor", check_sensor);
    NODE_SET_PROTOTYPE_METHOD(tpl, "heater_on", heater_on);
    NODE_SET_PROTOTYPE_METHOD(tpl, "heater_off", heater_off);
    NODE_SET_PROTOTYPE_METHOD(tpl, "get_rh", get_rh);
    NODE_SET_PROTOTYPE_METHOD(tpl, "get_temp", get_temp);

    constructor.Reset(isolate, tpl->GetFunction());
    exports->Set(String::NewFromUtf8(isolate, "TH02"), tpl->GetFunction());
}


void TH02::New(const FunctionCallbackInfo<Value>& args) {
    Isolate* isolate = args.GetIsolate();

    if (!args.IsConstructCall()) {
        isolate->ThrowException(Exception::TypeError(String::NewFromUtf8(isolate, "pls call with new")));
        return;
    }

	try {
		TH02* obj = new TH02();
		obj->Wrap(args.This());
		
		args.GetReturnValue().Set(args.This());
    } catch (std::exception & e) {
        isolate->ThrowException(Exception::Error(String::NewFromUtf8(isolate, e.what())));
        return;
    }
}

void TH02::check_sensor(const FunctionCallbackInfo<Value>& args) {
    Isolate* isolate = args.GetIsolate();

    if (args.Length() != 0) {
        isolate->ThrowException(Exception::TypeError(String::NewFromUtf8(isolate, "Wrong arguments")));
        return;
    }

    TH02* self = ObjectWrap::Unwrap<TH02>(args.Holder());
    try {
        self->th02obj.check_sensor();
    } catch (std::exception & e) {
        isolate->ThrowException(Exception::Error(String::NewFromUtf8(isolate, e.what())));
        return;
    }
}


void TH02::heater_on(const FunctionCallbackInfo<Value>& args) {
    Isolate* isolate = args.GetIsolate();

    if (args.Length() != 0) {
        isolate->ThrowException(Exception::TypeError(String::NewFromUtf8(isolate, "Wrong arguments")));
        return;
    }

    TH02* self = ObjectWrap::Unwrap<TH02>(args.Holder());
    try {
        self->th02obj.heater_on();
    } catch (std::exception & e) {
        isolate->ThrowException(Exception::Error(String::NewFromUtf8(isolate, e.what())));
        return;
    }
}


void TH02::heater_off(const FunctionCallbackInfo<Value>& args) {
    Isolate* isolate = args.GetIsolate();

    if (args.Length() != 0) {
        isolate->ThrowException(Exception::TypeError(String::NewFromUtf8(isolate, "Wrong arguments")));
        return;
    }

    TH02* self = ObjectWrap::Unwrap<TH02>(args.Holder());
    try {
        self->th02obj.heater_off();
    } catch (std::exception & e) {
        isolate->ThrowException(Exception::Error(String::NewFromUtf8(isolate, e.what())));
        return;
    }
}


void TH02::get_rh(const FunctionCallbackInfo<Value>& args) {
    Isolate* isolate = args.GetIsolate();

    if (args.Length() != 1 && !args[0]->IsNumber()) {
        isolate->ThrowException(Exception::TypeError(String::NewFromUtf8(isolate, "Wrong arguments")));
        return;
    }

    try {
        TH02* self = ObjectWrap::Unwrap<TH02>(args.Holder());
        double ret = self->th02obj.get_rh(args[0]->NumberValue());
        args.GetReturnValue().Set(Number::New(isolate, ret));
    } catch (std::exception & e) {
        isolate->ThrowException(Exception::Error(String::NewFromUtf8(isolate, e.what())));
        return;
    }
}


void TH02::get_temp(const FunctionCallbackInfo<Value>& args) {
    Isolate* isolate = args.GetIsolate();

    if (args.Length() != 1 && !args[0]->IsNumber()) {
        isolate->ThrowException(Exception::TypeError(String::NewFromUtf8(isolate, "Wrong arguments")));
        return;
    }

    try {
        TH02* self = ObjectWrap::Unwrap<TH02>(args.Holder());
        double ret = self->th02obj.get_temp(args[0]->NumberValue());
        args.GetReturnValue().Set(Number::New(isolate, ret));
    } catch (std::exception & e) {
        isolate->ThrowException(Exception::Error(String::NewFromUtf8(isolate, e.what())));
        return;
    }
}


}
