#include <iostream>
#include <unistd.h>
#include <time.h>
#include "mraa.hpp"
#include "led_interface.hpp"
#include "th02.hpp"
#include "yitran_rs.hpp"
//-------------------------------------------------------------------------------------------------
int application_rs (const short pylon_id, const char measurement_required_flag) {

	//////////////////////////////////////
	// PLC network management resources //
	//////////////////////////////////////
	yitran_host::yitran_rs rs(false);
	unsigned network_size = 2;

	enum state_t {
		init,
		wait_for_low_level_nc_connection,
		send_login_message,
		wait_for_login_approval,
		do_application_logic
	};
	state_t state = init;

	yitran_host::runtime_indication recent_runtime_indication = yitran_host::unknown_indication;
	unsigned char login_message[3];
	unsigned login_trials = 0;



	/////////////////////////////////
	// Application logic resources //
	/////////////////////////////////
	led_interface::led_interface ledif;
	th02::th02 trh;

	unsigned char measurement_data[256];
	for ( int i = 0; i < 256; ++i ) measurement_data[i] = 0;
	float tmp = 0;

	while (1) {
		std::cout << "Connecting to Szasza's controller..." << std::endl;
		try {
			ledif.check_connection();
			std::cout << "Connected to Szasza's controller" << std::endl;
			break;
		} catch (std::exception& e) {
			std::cout << e.what() << std::endl;
		}
		sleep(1);
	}

	while (1) {
		std::cout << "Connecting to TH02 sensor..." << std::endl;
		try {
			trh.check_sensor();
			std::cout << "Connected to TH02 sensor" << std::endl;
			break;
		} catch (std::exception& e) {
			std::cout << e.what() << std::endl;
		}
		sleep(1);
	}



	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



	while (1) {
		switch (state) {
			case init:
				std::cout << "Initialize RS..." << std::endl;
				rs.reset_and_go_online(network_size);
				std::cout << "RS initialized." << std::endl;
				std::cout << "Waiting for Connected to NC indication." << std::endl;
				state = wait_for_low_level_nc_connection;
				break;

				//-----------------------------------------------------------------------------------------------------------------------------------------------------------

			case wait_for_low_level_nc_connection:
				try {
					recent_runtime_indication = rs.wait_for_runtime_indication(30000);
					if ( recent_runtime_indication == yitran_host::connected_to_nc ) {
						std::cout << "Connected to NC indication received. Node ID: " << rs.get_node_id(2000) << std::endl;
						std::cout << "Sending login message to NC..." << std::endl;
						state = send_login_message;
					}
				} catch (std::exception& e) {
					std::cout << "Connected to NC indication missing... Re-initialize and retry..." << std::endl;
					state = init;
				}
				break;

				//-----------------------------------------------------------------------------------------------------------------------------------------------------------

			case send_login_message:
				login_message[0] = 0x7E;
				login_message[1] = (0x00FF & rs.get_node_id(5000));
				login_message[2] = (0xFF00 & rs.get_node_id(5000)) >> 8;
				rs.non_acknowledged_packet_tx(1, login_message, 3);
				state = wait_for_login_approval;
				break;

				//-----------------------------------------------------------------------------------------------------------------------------------------------------------

			case wait_for_login_approval:
				try {
					recent_runtime_indication = rs.wait_for_runtime_indication(2000);
					if ( recent_runtime_indication == yitran_host::rx_intranetworking_packet ) {
						rs.decode_rx_intranetworking_packet_indication();
						if ( rs.recently_received_packet.user_payload[0] == 0x7E ) {
							state = do_application_logic;
						} else {
							// wait for the approval a bit more...
						}
					} else {
						// wait for the approval a bit more...
					}
				} catch (std::exception& e) {
					++login_trials;
					if ( login_trials == 10 ) {
						login_trials = 0;
						std::cout << "Login attempt failed 10 times. Re-initialize..." << std::endl;
						state = init;
					} else {
						std::cout << "Login approve timed out... Retry..." << std::endl;
						state = send_login_message;
					}
				}
				break;

				//-----------------------------------------------------------------------------------------------------------------------------------------------------------

			case do_application_logic: // Check for HK DAQ request or login confirmation request or disconnect from NC indication.

				try {
					recent_runtime_indication = rs.wait_for_runtime_indication(60000);
				} catch (std::exception& e) {
					std::cout << "NC is suspiciously silent. Re-initialize..." << std::endl;
					state = init;
					break;
				}

				switch ( recent_runtime_indication ) {
					case yitran_host::rx_intranetworking_packet:
						rs.decode_rx_intranetworking_packet_indication();

						switch (rs.recently_received_packet.user_payload[0] ) {
							case 0x7E: // It's a login confirmation request.
								// Send a login message.
								//std::cout << "Login confirmation request received. Sending login confirmation..." << std::endl;
								state = send_login_message;
								break;

							case 0x80: // It's a data exchange transaction request
								// Check, whether the luminance shall be adjusted.
								if ( rs.recently_received_packet.user_payload[1] == 0x01 ) {
									// Adjust luminance
									ledif.set_luminance(rs.recently_received_packet.user_payload[2]);
									std::cout << "Luminance adjusted: " << 100 * rs.recently_received_packet.user_payload[2] / 255 << "%" << std::endl;
								}

								measurement_data[0] = 0x80;

								// Collecting measurement data...
								// Reading LED string voltages
								tmp = ledif.get_u(1);
								measurement_data[1] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
								measurement_data[2] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
								measurement_data[3] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
								measurement_data[4] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));

								tmp = ledif.get_u(2);
								measurement_data[5] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
								measurement_data[6] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
								measurement_data[7] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
								measurement_data[8] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));

								tmp = ledif.get_u(3);
								measurement_data[9] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
								measurement_data[10] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
								measurement_data[11] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
								measurement_data[12] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));

								tmp = ledif.get_u(4);
								measurement_data[13] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
								measurement_data[14] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
								measurement_data[15] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
								measurement_data[16] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));

								// Reading LED string currents
								tmp = ledif.get_i(1);
								measurement_data[17] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
								measurement_data[18] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
								measurement_data[19] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
								measurement_data[20] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));

								tmp = ledif.get_i(2);
								measurement_data[21] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
								measurement_data[22] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
								measurement_data[23] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
								measurement_data[24] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));

								tmp = ledif.get_i(3);
								measurement_data[25] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
								measurement_data[26] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
								measurement_data[27] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
								measurement_data[28] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));

								tmp = ledif.get_i(4);
								measurement_data[29] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
								measurement_data[30] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
								measurement_data[31] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
								measurement_data[32] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));

								try {
									tmp = trh.get_temp(1000);
									measurement_data[33] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
									measurement_data[34] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
									measurement_data[35] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
									measurement_data[36] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));

									tmp = trh.get_rh(1000);
									measurement_data[37] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
									measurement_data[38] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
									measurement_data[39] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
									measurement_data[40] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));
								} catch (std::exception& e) {
									std::cout << e.what() << std::endl;
								}

								// Get the recent luminance
								measurement_data[41] = ledif.get_luminance();

								// Get Pylon ID and maintenance required flag
								measurement_data[42] = (pylon_id & 0xFF00) >> 8;	// MS byte
								measurement_data[43] = pylon_id & 0x00FF;			// LS byte
								measurement_data[44] = measurement_required_flag;

								// Send HK data back to the NC
								rs.acknowledged_packet_tx(1, measurement_data, 45, 2000);
								break;

							default:
								std::cout << "Unknown command discarded..." << std::endl;
								break;
						}

						break;

					case yitran_host::disconnected_from_nc:
						std::cout << "Disconnected from NC... Restart..." << std::endl;
						state = init;
						break;
					default:
						//...
						break;
				}

				break;

				//-----------------------------------------------------------------------------------------------------------------------------------------------------------

			default:
				//...
				break;
		}
	}

	return 0;

}
//-------------------------------------------------------------------------------------------------
