#include <iostream>
#include <unistd.h>
#include <time.h>
#include <set>
#include "mraa.hpp"
#include "yitran_host.hpp"
#include "yitran_nc.hpp"
#include "yitran_node_descriptor.hpp"
//-------------------------------------------------------------------------------------------------
float calculate_optimal_forward_current(float ambient_temperature) {
	//float ret_value = (0.003881554 * ambient_temperature * ambient_temperature) + (0.626921775 * ambient_temperature) + 477.6894677;
	return 20;
	//return ret_value;
}
//-------------------------------------------------------------------------------------------------
int disconnect_test_nc () {

	//////////////////////////////////////
	// PLC network management resources //
	//////////////////////////////////////
	yitran_host::yitran_nc nc(false);
	unsigned network_size = 2;

	enum state_t {
		init,
		do_application_logic,
		check_for_login_messages_from_new_rss,
		request_login_confirmations
	};
	state_t state = init;

	yitran_host::runtime_indication recent_runtime_indication = yitran_host::unknown_indication;
	short new_rs_id = 0;
	std::set<short> logged_in_rss;
	unsigned char login_request = 0x7E;



	/////////////////////////////////
	// Application logic resources //
	/////////////////////////////////
	//unsigned char full_hk_daq_request = 0x7F;
	unsigned char temp_request = 0x80;
	unsigned char current_request = 0x81;
	unsigned char luminance_adjust_request[2];	// [0x82 ; value]
	unsigned char luminance_request = 0x83;

	unsigned char luminance_value = 20;
	unsigned tmp = 0;
	float recent_temperature = 20;
	float recent_current = 150;
	unsigned char recent_luminance = 0;
	float optimal_forward_current = 0;



	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



	while (1) {
		switch (state) {
			case init:
				std::cout << "Initialize NC..." << std::endl;
				nc.reset_and_go_online(network_size);
				std::cout << "NC initialized. Do application logic..." << std::endl;
				state = do_application_logic;
				break;

				//---------------------------------------------------------------------------------

			case do_application_logic:

				std::cout << std::endl << std::endl;
				for(std::set<short>::iterator p = logged_in_rss.begin(); p != logged_in_rss.end(); ++p) {
					try {

						// START APPLICATION LOGIC
						// Acquire recent luminance
						nc.acknowledged_packet_tx((*p), &luminance_request, 1, 2000);
						recent_runtime_indication = nc.wait_for_runtime_indication(10000);
						if ( recent_runtime_indication == yitran_host::rx_intranetworking_packet ) {
							nc.decode_rx_intranetworking_packet_indication();
							if ( nc.recently_received_packet.user_payload[0] == 0x83 ) {
								// It is a luminance request reply.
								recent_luminance = nc.recently_received_packet.user_payload[1];
								std::cout << "RS #" << (*p) << " luminance: " << (unsigned)recent_luminance << std::endl;
							} else {
								std::cout << "It's not a luminance request reply. Skip RS..." << std::endl;
								continue;
							}
						} else {
							std::cout << "It's not an RX intranetworking packet. Skip RS..." << std::endl;
							continue;
						}

						// Acquire recent temperature
						nc.acknowledged_packet_tx((*p), &temp_request, 1, 2000);
						recent_runtime_indication = nc.wait_for_runtime_indication(10000);
						if ( recent_runtime_indication == yitran_host::rx_intranetworking_packet ) {
							nc.decode_rx_intranetworking_packet_indication();
							if ( nc.recently_received_packet.user_payload[0] == 0x80 ) {
								// It is a temperature request reply.
								tmp = 0;
								tmp |= (nc.recently_received_packet.user_payload[1] << 24);
								tmp |= (nc.recently_received_packet.user_payload[2] << 16);
								tmp |= (nc.recently_received_packet.user_payload[3] << 8);
								tmp |= nc.recently_received_packet.user_payload[4];
								if ( *reinterpret_cast<float*>(&tmp) > recent_temperature + 30 || *reinterpret_cast<float*>(&tmp) < recent_temperature - 30 ) {
									// measurement probably failed...
									// don't refresh the recent_temperature
								} else {
									recent_temperature = *reinterpret_cast<float*>(&tmp);
								}
								std::cout << "RS #" << (*p) << " temperature: " <<  recent_temperature << " Celsius" << std::endl;

							} else {
								std::cout << "It's not a temperature request reply. Skip RS..." << std::endl;
								continue;
							}

						} else {
							std::cout << "It's not an RX intranetworking packet. Skip RS..." << std::endl;
							continue;
						}

						// Acquire recent current
						nc.acknowledged_packet_tx((*p), &current_request, 1, 2000);
						recent_runtime_indication = nc.wait_for_runtime_indication(10000);
						if ( recent_runtime_indication == yitran_host::rx_intranetworking_packet ) {
							nc.decode_rx_intranetworking_packet_indication();
							if ( nc.recently_received_packet.user_payload[0] == 0x81 ) {
								// It is a current request reply.
								tmp = 0;
								tmp |= (nc.recently_received_packet.user_payload[1] << 24);
								tmp |= (nc.recently_received_packet.user_payload[2] << 16);
								tmp |= (nc.recently_received_packet.user_payload[3] << 8);
								tmp |= nc.recently_received_packet.user_payload[4];
								if ( *reinterpret_cast<float*>(&tmp) > recent_current + 250 || *reinterpret_cast<float*>(&tmp) < recent_current - 250 ) {
									// measurement probably failed...
									// don't refresh the recent_current
								} else {
									recent_current = *reinterpret_cast<float*>(&tmp);
								}
								std::cout << "RS #" << (*p) << " current: " << recent_current << " mA" << std::endl;
								recent_current = *reinterpret_cast<float*>(&tmp);
							} else {
								std::cout << "It's not a current request reply. Skip RS..." << std::endl;
								continue;
							}
						} else {
							std::cout << "It's not an RX intranetworking packet. Skip RS..." << std::endl;
							continue;
						}

						// Calculate optimal forward current
						optimal_forward_current = calculate_optimal_forward_current(recent_temperature);
						std::cout << "Optimal forward current: " << optimal_forward_current << " mA" << std::endl;
						if ( optimal_forward_current < recent_current ) {
							std::cout << "Decrease luminance..." << std::endl;
							if ( recent_luminance > 20 ) luminance_value = recent_luminance - 10;
							else luminance_value = recent_luminance;
						} else {
							std::cout << "Increase luminance..." << std::endl;
							if ( recent_luminance < 244 ) luminance_value = recent_luminance + 10;
							else luminance_value = recent_luminance;
						}

						// Send new luminance value
						luminance_adjust_request[0] = 0x82;
						luminance_adjust_request[1] = luminance_value;
						nc.acknowledged_packet_tx((*p), luminance_adjust_request, 2, 2000);
						std::cout << "Luminance value adjusted to " << (unsigned)luminance_value << std::endl;

						// END OF APPLICATION LOGIC

					} catch (std::exception& e) {
						std::cout << "RS #" << (*p) << " application logic failed... continue..." << std::endl;
						continue;
					}
				}
				std::cout << std::endl << std::endl;

				if ( logged_in_rss.size() < network_size ) {
					std::cout << "Waiting for new RSs..." << std::endl;
					state = check_for_login_messages_from_new_rss;
				} else {
					state = request_login_confirmations;
				}
				break;

				//---------------------------------------------------------------------------------

			case check_for_login_messages_from_new_rss:

				if ( logged_in_rss.size() == 0 ) {
					try {
						recent_runtime_indication = nc.wait_for_runtime_indication(60000);
					} catch (std::exception& e) {
						std::cout << "RSs seem to be silent. My modem is possibly fucked up... Better to re-initialize..." << std::endl;
						state = init;
						break;
					}
				} else {
					try {
						recent_runtime_indication = nc.wait_for_runtime_indication(5000);
					} catch (std::exception& e) {
						state = request_login_confirmations;
						break;
					}
				}

				if ( recent_runtime_indication == yitran_host::rx_intranetworking_packet ) {
					nc.decode_rx_intranetworking_packet_indication();
					if ( nc.recently_received_packet.user_payload_size == 3 && nc.recently_received_packet.user_payload[0] == 0x7E ) {
						new_rs_id = (nc.recently_received_packet.user_payload[2] << 8) | nc.recently_received_packet.user_payload[1];
						std::cout << "Login message received from " << new_rs_id << ". Inserting RS into logged_in_rss..." << std::endl;
						logged_in_rss.insert(new_rs_id);
						std::cout << "Sending approval to " << new_rs_id << std::endl;
						nc.acknowledged_packet_tx(new_rs_id, &login_request, 1, 5000);
					} else {
						// It's not a login message... skip...
					}
				} else {
					// Only RX Intranetworking Packets are handled.
				}

				break;

				//---------------------------------------------------------------------------------

			case request_login_confirmations:
				// Send login requests to the RSs already logged in and refresh the logged_in_rss set according to the responses received.
				// If a login message is received, which is not sent by an RS already logged in, the message is discarded (and processed in the next cycle
				// in the check_for_login_messages_from_new_rss state).

				for(std::set<short>::iterator p = logged_in_rss.begin(); p != logged_in_rss.end(); ++p) {

					std::cout << "Sending login request to " << (*p) << "..." << std::endl;

					try {
						nc.acknowledged_packet_tx((*p), &login_request, 1, 5000);
						// Wait for response
						nc.wait_for_runtime_indication(5000);
						if ( recent_runtime_indication == yitran_host::rx_intranetworking_packet ) {
							nc.decode_rx_intranetworking_packet_indication();
							if ( nc.recently_received_packet.user_payload_size == 3 && nc.recently_received_packet.user_payload[0] == 0x7E ) {
								std::cout << "Login message received." << std::endl;
								if ( ((nc.recently_received_packet.user_payload[2] << 8) | nc.recently_received_packet.user_payload[1]) == (*p) ) {
									// Login confirmation received from the RS.
									std::cout << "Login confirmed by RS. Send acknowledge..." << std::endl;
									// Send login approve.
									nc.acknowledged_packet_tx((*p), &login_request, 1, 5000);
									continue;
								} else {
									std::cout << "Login message received from a new RS. Retry to receive login message confirmation from the same RS." << std::endl;
									--p; // TODO!!!! Mi van, ha p = 0?
								}
							} else {
								// It's not a login message... skip...
							}
						} else {
							// Only RX Intranetworking PAckets are handled.
						}
					} catch (std::exception& e) {
						std::cout << "Login request failed or confirmation timed out. Remove the RS from the logged_in_rss set." << std::endl;
						logged_in_rss.erase(p);
					}

				}

				// Print logged_in_rss
				std::cout << std::endl << "RSs logged in:" << std::endl;
				for(std::set<short>::iterator p = logged_in_rss.begin(); p != logged_in_rss.end(); ++p) {
					std::cout << (*p) << ", ";
				}
				std::cout << std::endl;

				sleep(5);

				// logged_in_rss set is refreshed. Go back to application logic
				state = do_application_logic;
				break;

				//---------------------------------------------------------------------------------

			//...

			default:
				//...
				break;
		}
	}

	return 0;

}
//-------------------------------------------------------------------------------------------------
