#include <iostream>
#include <unistd.h>
#include <time.h>
#include "mraa.hpp"
#include "led_interface.hpp"
#include "th02.hpp"
#include "yitran_rs.hpp"
//-------------------------------------------------------------------------------------------------
int full_test_rs () {

	unsigned char dali_led_linearization_array[256] = {
		 52, 77, 92,103,111,117,123,128,132,136,140,143,146,148,151,153,156,158,160,161,163,165,167,168,170,171,172,174,175,176,178,179,
		180,181,182,183,184,185,186,187,188,189,190,190,191,192,193,194,194,195,196,196,197,198,199,199,200,200,201,202,202,203,204,204,
		205,205,206,206,207,207,208,208,209,209,210,210,211,211,212,212,213,213,214,214,214,215,215,216,216,217,217,217,218,218,219,219,
		219,220,220,220,221,221,222,222,222,223,223,223,224,224,224,225,225,225,226,226,226,227,227,227,227,228,228,228,229,229,229,229,
		230,230,230,231,231,231,231,232,232,232,233,233,233,233,234,234,234,234,235,235,235,235,236,236,236,236,236,237,237,237,237,238,
		238,238,238,239,239,239,239,239,240,240,240,240,241,241,241,241,241,242,242,242,242,242,243,243,243,243,243,244,244,244,244,244,
		245,245,245,245,245,245,246,246,246,246,246,247,247,247,247,247,247,248,248,248,248,248,248,249,249,249,249,249,249,250,250,250,
		250,250,250,251,251,251,251,251,251,252,252,252,252,252,252,253,253,253,253,253,253,253,254,254,254,254,254,254,254,254,254,254
	};

	led_interface::led_interface ledif;
	th02::th02 trh;

	mraa::Gpio* uart2_reset_pin;
	uart2_reset_pin = new mraa::Gpio(15);	// GPIO165
	uart2_reset_pin->dir(mraa::DIR_OUT);

	uart2_reset_pin->write(0);
	sleep(2);
	uart2_reset_pin->write(1);

	while (1) {
		std::cout << "Connecting to Szasza's controller..." << std::endl;
		try {
			ledif.check_connection();
			std::cout << "Connected to Szasza's controller" << std::endl;
			break;
		} catch (std::exception& e) {
			std::cout << e.what() << std::endl;
		}
		sleep(2);
	}

	while (1) {
		std::cout << "Connecting to TH02 sensor..." << std::endl;
		try {
			trh.check_sensor();
			std::cout << "Connected to TH02 sensor" << std::endl;
			break;
		} catch (std::exception& e) {
			std::cout << e.what() << std::endl;
		}
		sleep(2);
	}

	try {

		// A Yitran RS host is created using the non-console UART (UART1).
		yitran_host::yitran_rs rs(false);

		while (1) {
			if ( !rs.initialize_and_login(5) ) {
				printf("FAIL: RS initialization failed -> RETRY\n");
			} else break;
		}

		if ( !rs.wait_for_data_traffic_enabled_indication(50000) ) {
			printf("FAIL: Waiting for \"Data Traffic Enabled\" indication timed out -> TERMINATE\n");
			return 0;
		}

		yitran_host::runtime_indication recent_runtime_indication = yitran_host::unknown_indication;
		unsigned char measurement_data[40];
		for ( int i = 0; i < 40; ++i ) measurement_data[i] = 0;
		float tmp = 0;
		while (1) {

			// Waiting for the new luminance value.
			while (1) {

				try {
					recent_runtime_indication = rs.wait_for_runtime_indication(5000);
				} catch (std::exception& e) {
					std::cout << e.what() << std::endl;
					std::cout << "RETRY" << std::endl;
					continue;
				}

				if ( recent_runtime_indication == yitran_host::rx_intranetworking_packet ) {
					rs.decode_rx_intranetworking_packet_indication();
					//printf("Packet received: Origin ID: %d, Final ID: %d, user payload size: %d\n", rs.recently_received_packet.origin_id, rs.recently_received_packet.final_id, rs.recently_received_packet.user_payload_size);
					//printf("User payload: ");
					//for ( unsigned i = 0; i < rs.recently_received_packet.user_payload_size; ++i ) printf("%d ", rs.recently_received_packet.user_payload[i]);
					//printf("\n");

					// Setting new luminance value...
					ledif.set_luminance(dali_led_linearization_array[rs.recently_received_packet.user_payload[0]]);
					std::cout << "Luminance adjusted: " << ((double)(unsigned)(rs.recently_received_packet.user_payload[0]) / 255) * 100 << " %" << std::endl;

					// Collecting measurement data...
					// Reading LED string #1 voltage
					tmp = ledif.get_u(1);
					measurement_data[0] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
					measurement_data[1] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
					measurement_data[2] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
					measurement_data[3] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));

					// The other LED strings are not present in the demonstrator.
					// The values are read from LED string #1
					measurement_data[4] = measurement_data[0];
					measurement_data[5] = measurement_data[1];
					measurement_data[6] = measurement_data[2];
					measurement_data[7] = measurement_data[3];
					measurement_data[8] = measurement_data[0];
					measurement_data[9] = measurement_data[1];
					measurement_data[10] = measurement_data[2];
					measurement_data[11] = measurement_data[3];
					measurement_data[12] = measurement_data[0];
					measurement_data[13] = measurement_data[1];
					measurement_data[14] = measurement_data[2];
					measurement_data[15] = measurement_data[3];

					// Reading LED string #1 current
					tmp = ledif.get_i(1);
					measurement_data[16] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
					measurement_data[17] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
					measurement_data[18] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
					measurement_data[19] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));

					// The other LED strings are not present in the demonstrator.
					// The values are read from LED string #1
					measurement_data[20] = measurement_data[16];
					measurement_data[21] = measurement_data[17];
					measurement_data[22] = measurement_data[18];
					measurement_data[23] = measurement_data[19];
					measurement_data[24] = measurement_data[16];
					measurement_data[25] = measurement_data[17];
					measurement_data[26] = measurement_data[18];
					measurement_data[27] = measurement_data[19];
					measurement_data[28] = measurement_data[16];
					measurement_data[29] = measurement_data[17];
					measurement_data[30] = measurement_data[18];
					measurement_data[31] = measurement_data[19];

					// Reading temperature
					try {
						tmp = trh.get_temp(1000);
						measurement_data[32] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
						measurement_data[33] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
						measurement_data[34] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
						measurement_data[35] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));

						// Reading humidity
						tmp = trh.get_rh(1000);
						measurement_data[36] = (0xFF000000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 24;
						measurement_data[37] = (0x00FF0000 & *(reinterpret_cast<unsigned*>(&tmp))) >> 16;
						measurement_data[38] = (0x0000FF00 & *(reinterpret_cast<unsigned*>(&tmp))) >> 8;
						measurement_data[39] = (0x000000FF & *(reinterpret_cast<unsigned*>(&tmp)));
					} catch (std::exception& e) {
						std::cout << e.what() << std::endl;
					}

					// Sending measurement data back
					if ( !rs.acknowledged_packet_tx(1, measurement_data, 40, 20000) ) {
						printf("FAIL: Packet TX FAILED -> TERMINATE\n");
						return 0;
					} //else printf("Measurement data sent successfully\n");


				} else continue;
			}

		}

		return 0;

	} catch (std::exception& e) {
		std::cout << e.what() << std::endl;
		std::cout << "TERMINATE" << std::endl;
		return 0;
	}

	return 0;

}
//-------------------------------------------------------------------------------------------------
