#ifndef _TH02_
#define _TH02_
//-------------------------------------------------------------------------------------------------
#include <iostream>
#include "mraa.hpp"
//-------------------------------------------------------------------------------------------------
namespace th02 {
	
	typedef enum command {
		heater_on,
		heater_off,
		get_rh,
		get_temp
	} command;
	
	
	
	class th02 {
	
		public:
			th02();
			~th02();



			/*
			 * The function checks, whether the TH02 sensor is accessible.
			 * If the sensor is not available, a sensor_not_responding_exception is thrown.
			 */
			void check_sensor();



			/*
			 * The function turns the internal heating facility of the TH02 sensor on.
			 */
			void heater_on();



			/*
			 * The function turns the internal heating facility of the TH02 sensor off.
			 */
			void heater_off();



			/*
			 * The function reads the relative humidity measured by the TH02 sensor.
			 * In case of timeout, a conversion_timed_out_exception is thrown.
			 */
			float get_rh(unsigned timeout_ms);



			/*
			 * The function reads the temperature measured by the TH02 sensor.
			 * In case of timeout, a conversion_timed_out_exception is thrown.
			 */
			float get_temp(unsigned timeout_ms);
			
		
		//private:
			mraa::I2c* i2c;
			float recent_rh;
			float recent_temp;
			
			unsigned char read_config();
			void write_config(unsigned char data);
			unsigned char read_id();
			unsigned char read_rdy();
			void read_data(unsigned char output_buffer[2]);
			
	};
	
	
	
	struct sensor_not_responding_exception : public std::exception {
		const char* what() const throw() { return "TH02 sensor doesn't respond!"; }
	};
	
	struct conversion_timed_out_exception : public std::exception {
		const char* what() const throw() { return "Conversion timed out!"; }
	};
	
}
//-------------------------------------------------------------------------------------------------
#endif // _TH02_
//-------------------------------------------------------------------------------------------------
