#include "yitran_host.hpp"
#include "yitran_nc.hpp"
//-------------------------------------------------------------------------------------------------
namespace yitran_host {
	yitran_nc::yitran_nc(bool use_console_port) : yitran_host(use_console_port) {
		for ( int i = 0; i < 256; ++i ) rs_address_table[i] = 0;
		number_of_connected_rss = 0;
	}
	yitran_nc::~yitran_nc() {}

	void yitran_nc::send_data_exchange_request(const unsigned destination_id, const unsigned char new_luminance) {
		unsigned char data_exchange_request[6];
		data_exchange_request[0] = 0x80;									// command header
		data_exchange_request[1] = (destination_id & 0xFF000000) >> 24;		// RS address byte #3 (MS)
		data_exchange_request[2] = (destination_id & 0x00FF0000) >> 16;		// RS address byte #2
		data_exchange_request[3] = (destination_id & 0x0000FF00) >> 8;		// RS address byte #1
		data_exchange_request[4] = (destination_id & 0x000000FF);			// RS address byte #0 (LS)
		data_exchange_request[5] = new_luminance;							// new luminance value
		acknowledged_packet_tx(0, data_exchange_request, 6, 2000);
	}

	rs_hk_data_set yitran_nc::decode_data_exchange_reply() {
		rs_hk_data_set ret_value;
		unsigned tmp = 0;
		if ( recently_received_packet.user_payload[0] == 0x81 ) {
			// It is a valid data exchange reply.
			// Decode the data exchange reply.
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
			// Decode pylon ID
			tmp |= (recently_received_packet.user_payload[1] << 24);
			tmp |= (recently_received_packet.user_payload[2] << 16);
			tmp |= (recently_received_packet.user_payload[3] << 8);
			tmp |= recently_received_packet.user_payload[4];
			ret_value.pylon_id = tmp;
			tmp = 0;
			// Decode U1
			tmp |= (recently_received_packet.user_payload[5] << 24);
			tmp |= (recently_received_packet.user_payload[6] << 16);
			tmp |= (recently_received_packet.user_payload[7] << 8);
			tmp |= recently_received_packet.user_payload[8];
			ret_value.u1 = *reinterpret_cast<float*>(&tmp);
			tmp = 0;
			// Decode U2
			tmp |= (recently_received_packet.user_payload[9] << 24);
			tmp |= (recently_received_packet.user_payload[10] << 16);
			tmp |= (recently_received_packet.user_payload[11] << 8);
			tmp |= recently_received_packet.user_payload[12];
			ret_value.u2 = *reinterpret_cast<float*>(&tmp);
			tmp = 0;
			// Decode U3
			tmp |= (recently_received_packet.user_payload[13] << 24);
			tmp |= (recently_received_packet.user_payload[14] << 16);
			tmp |= (recently_received_packet.user_payload[15] << 8);
			tmp |= recently_received_packet.user_payload[16];
			ret_value.u3 = *reinterpret_cast<float*>(&tmp);
			tmp = 0;
			// Decode U4
			tmp |= (recently_received_packet.user_payload[17] << 24);
			tmp |= (recently_received_packet.user_payload[18] << 16);
			tmp |= (recently_received_packet.user_payload[19] << 8);
			tmp |= recently_received_packet.user_payload[20];
			ret_value.u4 = *reinterpret_cast<float*>(&tmp);
			tmp = 0;
			// Decode I1
			tmp |= (recently_received_packet.user_payload[21] << 24);
			tmp |= (recently_received_packet.user_payload[22] << 16);
			tmp |= (recently_received_packet.user_payload[23] << 8);
			tmp |= recently_received_packet.user_payload[24];
			ret_value.i1 = *reinterpret_cast<float*>(&tmp);
			tmp = 0;
			// Decode I2
			tmp |= (recently_received_packet.user_payload[25] << 24);
			tmp |= (recently_received_packet.user_payload[26] << 16);
			tmp |= (recently_received_packet.user_payload[27] << 8);
			tmp |= recently_received_packet.user_payload[28];
			ret_value.i2 = *reinterpret_cast<float*>(&tmp);
			tmp = 0;
			// Decode I3
			tmp |= (recently_received_packet.user_payload[29] << 24);
			tmp |= (recently_received_packet.user_payload[30] << 16);
			tmp |= (recently_received_packet.user_payload[31] << 8);
			tmp |= recently_received_packet.user_payload[32];
			ret_value.i3 = *reinterpret_cast<float*>(&tmp);
			tmp = 0;
			// Decode I4
			tmp |= (recently_received_packet.user_payload[33] << 24);
			tmp |= (recently_received_packet.user_payload[34] << 16);
			tmp |= (recently_received_packet.user_payload[35] << 8);
			tmp |= recently_received_packet.user_payload[36];
			ret_value.i4 = *reinterpret_cast<float*>(&tmp);
			tmp = 0;
			// DECODE temperature
			tmp |= (recently_received_packet.user_payload[37] << 24);
			tmp |= (recently_received_packet.user_payload[38] << 16);
			tmp |= (recently_received_packet.user_payload[39] << 8);
			tmp |= recently_received_packet.user_payload[40];
			ret_value.temperature = *reinterpret_cast<float*>(&tmp);
			tmp = 0;
			// Decode humidity
			tmp |= (recently_received_packet.user_payload[41] << 24);
			tmp |= (recently_received_packet.user_payload[42] << 16);
			tmp |= (recently_received_packet.user_payload[43] << 8);
			tmp |= recently_received_packet.user_payload[44];
			ret_value.humidity = *reinterpret_cast<float*>(&tmp);
			tmp = 0;
#pragma GCC diagnostic pop
			// Decode luminance
			ret_value.luminance = recently_received_packet.user_payload[45];
			// Decode maintenance required flag
			return ret_value;
		} else {
			throw(yitran_nc_not_a_valid_data_exchange_reply_exception());
		}
	}
	bool yitran_nc::reset_and_go_online(unsigned network_size) {
		printf("Reseting Yitran module...\n");
		reset();
		printf("Waiting for \"BSP Welcome Message\"...\n");
		get_packet(20000);
		printf("\"BSP Welcome Message\" discarded\n");
		printf("Waiting for \"Reset Response\" indication...\n");
		while (1) {
			if ( wait_for_reset_response(20000) ) {
				printf("\"Reset Response\" indication received\n");
				break;
			}
			else {
				printf("FAIL: No \"Reset Response\" indication received\n");
			}
		}
		printf("Setting operation mode: NC...\n");
		if ( verified_set_nc_operation_mode(20000) ) printf("Operation mode selection verified\n");
		else {
			printf("FAIL: Operation mode selection failed\n");
			return false;
		}
		printf("Setting network size...\n");
		if ( verified_set_network_size(network_size, 20000)) printf("Network size set to %d\n", network_size);
		else {
			printf("FAIL: Network size setting failed\n");
			return false;
		}
		printf("Saving device parameters to NVM...\n");
		if ( acknowledged_save_device_parameters(20000) ) printf("Device parameters saved to NVM\n");
		else {
			printf("FAIL: Device parameter saving failed\n");
			return false;
		}
		printf("Reseting Yitran module...\n");
		reset();
		printf("Waiting for \"BSP Welcome Message\"...\n");
		get_packet(20000);
		printf("\"BSP Welcome Message\" discarded\n");
		printf("Waiting for \"Reset Response\" indication...\n");
		if ( wait_for_reset_response(20000) ) printf("\"Reset Response\" indication received\n");
		else {
			printf("FAIL: No \"Reset Response\" indication received\n");
			return false;
		}
		printf("GO ONLINE!...\n");
		if ( acknowledged_go_online(20000) ) printf("NC is ON-LINE!\n");
		else {
			printf("FAIL: Setting RS to on-line FAILED\n");
			return false;
		}
		printf("Waiting for \"Network ID Assigned\" indication...\n");
		if ( wait_for_network_id_assigned_indication(20000) ) printf("Network ID assigned\n");
		else {
			printf("Fail: No \"Network ID Assigned\" indication received\n");
			return false;
		}
		printf("NC ready!\n");
		return true;
	}
	bool yitran_nc::initialize_and_establish_a_network(unsigned char initial_number_of_rss, unsigned network_size) {
		printf("Reseting Yitran module...\n");
		reset();
		printf("Waiting for \"BSP Welcome Message\"...\n");
		get_packet(20000);
		printf("\"BSP Welcome Message\" discarded\n");
		printf("Waiting for \"Reset Response\" indication...\n");
		if ( wait_for_reset_response(20000) ) printf("\"Reset Response\" indication received\n");
		else {
			printf("FAIL: No \"Reset Response\" indication received\n");
			return false;
		}
		printf("Setting operation mode: NC...\n");
		if ( verified_set_nc_operation_mode(20000) ) printf("Operation mode selection verified\n");
		else {
			printf("FAIL: Operation mode selection failed\n");
			return false;
		}
		printf("Setting network size...\n");
		if ( verified_set_network_size(network_size, 20000)) printf("Network size set to %d\n", network_size);
		else {
			printf("FAIL: Network size setting failed\n");
			return false;
		}
		printf("Saving device parameters to NVM...\n");
		if ( acknowledged_save_device_parameters(20000) ) printf("Device parameters saved to NVM\n");
		else {
			printf("FAIL: Device parameter saving failed\n");
			return false;
		}
		printf("Reseting Yitran module...\n");
		reset();
		printf("Waiting for \"BSP Welcome Message\"...\n");
		get_packet(20000);
		printf("\"BSP Welcome Message\" discarded\n");
		printf("Waiting for \"Reset Response\" indication...\n");
		if ( wait_for_reset_response(20000) ) printf("\"Reset Response\" indication received\n");
		else {
			printf("FAIL: No \"Reset Response\" indication received\n");
			return false;
		}
		printf("GO ONLINE!...\n");
		if ( acknowledged_go_online(20000) ) printf("NC is ON-LINE!\n");
		else {
			printf("FAIL: Setting RS to on-line FAILED\n");
			return false;
		}
		printf("Waiting for \"Network ID Assigned\" indication...\n");
		if ( wait_for_network_id_assigned_indication(20000) ) printf("Network ID assigned\n");
		else {
			printf("Fail: No \"Network ID Assigned\" indication received\n");
			return false;
		}
		runtime_indication recent_runtime_indication = unknown_indication;
		short recently_received_node_id = 0;
		unsigned char approval = 0x7E;
		for ( int i = 0; i < 50; ++i ) { // After 50 invalid indications, the network creation is considered unsuccessful.
			while (1) {
				printf("Waiting for RSs to login...\n");
				if ( (recent_runtime_indication = wait_for_runtime_indication(150000)) == rx_intranetworking_packet ) {
					decode_rx_intranetworking_packet_indication();
					printf("Packet received: Origin ID: %d, Final ID: %d, user payload size: %d\n", recently_received_packet.origin_id, recently_received_packet.final_id, recently_received_packet.user_payload_size);
					printf("User payload: ");
					for ( unsigned i = 0; i < recently_received_packet.user_payload_size; ++i ) printf("%d ", recently_received_packet.user_payload[i]);
					printf("\n");
					if ( recently_received_packet.user_payload_size == 3 && recently_received_packet.user_payload[0] == 0x7E ) {
						recently_received_node_id = (recently_received_packet.user_payload[2] << 8) | recently_received_packet.user_payload[1];
						printf("Login message received from %d\n", recently_received_node_id);
						printf("Sending approval to %d...\n", recently_received_node_id);
						if ( !acknowledged_packet_tx(recently_received_node_id, &approval, 1, 20000) ) {
							printf("FAIL: Packet TX FAILED\n");
							break;
						}
						printf("Packet sent successfully to %d\n", recently_received_node_id);
						printf("Saving node ID to address table...\n");
						rs_address_table[number_of_connected_rss] = recently_received_node_id;
						++number_of_connected_rss;
						if ( number_of_connected_rss == initial_number_of_rss ) {
							printf("RSs logged in, network established with %d RSs\n", initial_number_of_rss);
							return true;
						}
					} else {
						printf("RX packet discarded: not a login message\n");
						break;
					}
				} else {
					printf("Runtime indication discarded (opcode: %d)\n", recent_runtime_indication);
					break;
				}
			}
		}
		printf("FAIL: Establishing network failed: too many invalid indications received\n");
		return false;
	}
	void yitran_nc::send_data_traffic_enabled_indication() {
		unsigned char data_traffic_enabled_indication = 0x7F;
		for ( unsigned int i = 0; i < number_of_connected_rss; ++i ) {
			if ( !acknowledged_packet_tx(rs_address_table[i], &data_traffic_enabled_indication, 1, 20000) ) {
				printf("FAIL: Packet TX to %d FAILED\n", i);
			} else {
				printf("Data traffic enabled indication sent to %d\n", rs_address_table[i]);
			}
		}
		printf("Data traffic enabled indications sent\n");
		return;
	}
	void yitran_nc::decode_connectivity_status_with_rs_indication(yitran_node_descriptor& nd) {
		nd.node_logical_id = (recently_received_packet.payload[1] << 8) | recently_received_packet.payload[0];
		nd.connectivity_status = recently_received_packet.payload[18];
	}
	void yitran_nc::decode_new_connection_to_nc_indication(yitran_node_descriptor& nd) {
		nd.node_logical_id = (recently_received_packet.payload[1] << 8) | recently_received_packet.payload[0];
		nd.parent_logical_id = (recently_received_packet.payload[3] << 8) | recently_received_packet.payload[2];
	}
	bool yitran_nc::wait_for_network_id_assigned_indication(unsigned timeout_ms) {
		get_packet(timeout_ms);
		if ( recently_received_packet.command_start == 0xca && recently_received_packet.opcode == 0xbf ) return true;
		else {
			printf("wait_for_network_id_assigned_indication: Packet discarded\n");
			return false;
		}
	}
	bool yitran_nc::wait_for_new_connection_to_nc_indication(yitran_node_descriptor& nd, unsigned timeout_ms) {
		get_packet(timeout_ms);
		if ( recently_received_packet.command_start == 0xca && recently_received_packet.opcode == 0xbe ) {
			nd.node_logical_id = (recently_received_packet.payload[1] << 8) | recently_received_packet.payload[0];
			nd.parent_logical_id = (recently_received_packet.payload[3] << 8) | recently_received_packet.payload[2];
			nd.connectivity_status = 1;
			return true;
		}
		else {
			printf("wait_for_new_connection_to_nc_indication: Packet discarded\n");
			return false;
		}
	}
	bool yitran_nc::acknowledged_set_nc_operation_mode(unsigned timeout_ms) {
		set_nc_operation_mode();
		get_packet(timeout_ms);
		if ( recently_received_packet.payload[0] == 0x00 ) return false;
		else return true;
	}
	bool yitran_nc::verified_set_nc_operation_mode(unsigned timeout_ms) {
		if ( !acknowledged_set_nc_operation_mode(timeout_ms) ) return false;
		if ( get_operation_mode(timeout_ms) ) return true;
		else return false;
	}
	bool yitran_nc::acknowledged_set_nc_database_size(unsigned char size, unsigned timeout_ms) {
		set_nc_database_size(size);
		get_packet(timeout_ms);
		if ( recently_received_packet.payload[0] == 0x00 ) return false;
		else return true;
	}
	bool yitran_nc::verified_set_nc_database_size(unsigned char size, unsigned timeout_ms) {
		if ( !acknowledged_set_nc_database_size(size, timeout_ms) ) return false;
		if ( get_nc_database_size(timeout_ms) != size ) return false;
		else return true;
	}
	unsigned char yitran_nc::get_nc_database_size(unsigned timeout_ms) {
		yitran_packet p;
		p.command_start = 0xca;
		p.payload_size = 5;
		p.type = 0x00;
		p.opcode = 0x42;
		p.payload[0] = 0x06; // table type
		p.payload[1] = 0x5b; // index LSB
		p.payload[2] = 0x00; // index MSB
		p.payload[3] = 0x01; // number of parameters to read LSB
		p.payload[4] = 0x00; // number of parameters to read MSB
		send_packet(p);
		// wait for response packet
		get_packet(timeout_ms);
		return recently_received_packet.payload[1];
	}
	unsigned char yitran_nc::get_current_nc_database_size(unsigned timeout_ms) {
		yitran_packet p;
		p.command_start = 0xca;
		p.payload_size = 1;
		p.type = 0x00;
		p.opcode = 0x65;
		p.payload[0] = 0x01;
		send_packet(p);
		// wait for response packet
		get_packet(timeout_ms);
		return ((recently_received_packet.payload[4] << 8) | recently_received_packet.payload[3]);
	}
	bool yitran_nc::get_node_information(unsigned char node_index, yitran_node_descriptor& nd, unsigned timeout_ms) {
		// Check NC database index validity:
		if ( node_index > get_nc_database_size(timeout_ms) ) throw(invalid_nc_database_index_exception());
		yitran_packet p;
		p.command_start = 0xca;
		p.payload_size = 3;
		p.type = 0x00;
		p.opcode = 0x69;
		p.payload[0] = 0x00; // query key type: index in NC database
		p.payload[1] = node_index; // node index in the NC database LSB
		p.payload[2] = 0x00; // node index in the NC database MSB
		send_packet(p);
		// wait for response packet
		get_packet(timeout_ms);
		if ( recently_received_packet.payload[0] == 0x00 ) return false;
		else {
			nd.node_logical_id = (recently_received_packet.payload[2] << 8) | recently_received_packet.payload[1];
			nd.parent_logical_id = (recently_received_packet.payload[4] << 8) | recently_received_packet.payload[3];
			nd.connectivity_status = recently_received_packet.payload[21];
			return true;
		}
	}
}
//-------------------------------------------------------------------------------------------------
