#include "yitran_host.hpp"
#include "yitran_rs.hpp"
//-------------------------------------------------------------------------------------------------
namespace yitran_host {

	yitran_rs::yitran_rs(bool use_console_port) : yitran_host(use_console_port) {}



	yitran_rs::~yitran_rs() {}


	bool yitran_rs::reset_and_go_online(unsigned network_size) {
		printf("Reseting Yitran module...\n");
		reset();
		printf("Waiting for \"BSP Welcome Message\"...\n");
		get_packet(20000);
		printf("\"BSP Welcome Message\" discarded\n");

		printf("Waiting for \"Reset Response\" indication...\n");
		while (1) {
			if ( wait_for_reset_response(20000) ) {
				printf("\"Reset Response\" indication received\n");
				break;
			}
			else {
				printf("FAIL: No \"Reset Response\" indication received\n");
			}
		}

		printf("Setting operation mode: RS...\n");
		if ( verified_set_rs_operation_mode(20000) ) printf("Operation mode selection verified\n");
		else {
			printf("FAIL: Operation mode selection failed\n");
			return false;
		}

		printf("Setting network size...\n");
		if ( verified_set_network_size(network_size, 20000)) printf("Network size set to %d\n", network_size);
		else {
			printf("FAIL: Network size setting failed\n");
			return false;
		}

		printf("Saving device parameters to NVM...\n");
		if ( acknowledged_save_device_parameters(20000) ) printf("Device parameters saved to NVM\n");
		else {
			printf("FAIL: Device parameter saving failed\n");
			return false;
		}

		printf("Reseting Yitran module...\n");
		reset();

		printf("Waiting for \"BSP Welcome Message\"...\n");
		get_packet(20000);
		printf("\"BSP Welcome Message\" discarded\n");

		printf("Waiting for \"Reset Response\" indication...\n");
		if ( wait_for_reset_response(20000) ) printf("\"Reset Response\" indication received\n");
		else {
			printf("FAIL: No \"Reset Response\" indication received\n");
			return false;
		}

		printf("GO ONLINE!...\n");
		if ( acknowledged_go_online(20000) ) printf("RS is ON-LINE!\n");
		else {
			printf("FAIL: Setting RS to on-line FAILED\n");
			return false;
		}

		printf("RS ready!\n");
		return true;
	}



	bool yitran_rs::initialize_and_login(unsigned char network_size) {

		printf("Reseting Yitran module...\n");
		reset();
		printf("Waiting for \"BSP Welcome Message\"...\n");
		get_packet(20000);
		printf("\"BSP Welcome Message\" discarded\n");

		printf("Waiting for \"Reset Response\" indication...\n");
		if ( wait_for_reset_response(20000) ) printf("\"Reset Response\" indication received\n");
		else {
			printf("FAIL: No \"Reset Response\" indication received\n");
			return false;
		}

		printf("Setting operation mode: RS...\n");
		if ( verified_set_rs_operation_mode(20000) ) printf("Operation mode selection verified\n");
		else {
			printf("FAIL: Operation mode selection failed\n");
			return false;
		}

		printf("Setting network size...\n");
		if ( verified_set_network_size(network_size, 20000)) printf("Network size set to %d\n", network_size);
		else {
			printf("FAIL: Network size setting failed\n");
			return false;
		}

		printf("Saving device parameters to NVM...\n");
		if ( acknowledged_save_device_parameters(20000) ) printf("Device parameters saved to NVM\n");
		else {
			printf("FAIL: Device parameter saving failed\n");
			return false;
		}

		printf("Reseting Yitran module...\n");
		reset();

		printf("Waiting for \"BSP Welcome Message\"...\n");
		get_packet(20000);
		printf("\"BSP Welcome Message\" discarded\n");

		printf("Waiting for \"Reset Response\" indication...\n");
		if ( wait_for_reset_response(20000) ) printf("\"Reset Response\" indication received\n");
		else {
			printf("FAIL: No \"Reset Response\" indication received\n");
			return false;
		}

		printf("GO ONLINE!...\n");
		if ( acknowledged_go_online(20000) ) printf("RS is ON-LINE!\n");
		else {
			printf("FAIL: Setting RS to on-line FAILED\n");
			return false;
		}

		printf("Waiting for \"Connected to NC\" indication...\n");
		try {
			wait_for_connected_to_nc_indication(150000);
			printf("RS connected to NC\n");
		} catch (std::exception& e) {
			printf("FAIL: Connection attempt timed out\n");
			return false;
		}
		printf("RS Connected to NC\n");
		printf("Node ID: %d\n", get_node_id(20000));

		printf("Sending login message to NC...\n");
		unsigned char login_message[3];
		login_message[0] = 0x7E;
		login_message[1] = (0x00FF & get_node_id(20000));
		login_message[2] = (0xFF00 & get_node_id(20000)) >> 8;
		if ( !acknowledged_packet_tx(1, login_message, 3, 20000) ) {
			printf("FAIL: Transferring login message FAILED\n");
			return false;
		}
		printf("Login message sent successfully to NC\n");

		printf("Waiting for NC approval...\n");
		runtime_indication recent_runtime_indication = unknown_indication;
		for (int i = 0; i < 10; ++i) { // After 10 invalid indications, the connection is considered unsuccessful.
			if ( (recent_runtime_indication = wait_for_runtime_indication(50000)) == rx_intranetworking_packet ) {
				decode_rx_intranetworking_packet_indication();
				printf("Packet received: Origin ID: %d, Final ID: %d, user payload size: %d\n", recently_received_packet.origin_id, recently_received_packet.final_id, recently_received_packet.user_payload_size);
				printf("User payload: ");
				for ( unsigned i = 0; i < recently_received_packet.user_payload_size; ++i ) printf("%d ", recently_received_packet.user_payload[i]);
				if ( recently_received_packet.user_payload[0] == 0x7E ) {
					printf("\n");
					printf("Login approved by NC\n");
					return true;
				} else {
					printf("Invalid NC approval...\n");
					continue;
				}
			} else printf("Runtime indication discarded (opcode: %d):\n", recent_runtime_indication);
		}

		printf("FAIL: Login not approved by NC\n");
		return false;

	}



	bool yitran_rs::wait_for_data_traffic_enabled_indication(unsigned timeout_ms) {
		printf("Waiting for \"Data Traffic Enabled\" indication...\n");
		runtime_indication recent_runtime_indication = unknown_indication;
		while (1) {
			if ( (recent_runtime_indication = wait_for_runtime_indication(timeout_ms)) == rx_intranetworking_packet ) {
				decode_rx_intranetworking_packet_indication();
				printf("Packet received: Origin ID: %d, Final ID: %d, user payload size: %d\n", recently_received_packet.origin_id, recently_received_packet.final_id, recently_received_packet.user_payload_size);
				printf("User payload: ");
				for ( unsigned i = 0; i < recently_received_packet.user_payload_size; ++i ) printf("%d ", recently_received_packet.user_payload[i]);
				if ( recently_received_packet.user_payload[0] == 0x7F ) {
					printf("\n");
					printf("Data traffic enabled by NC\n");
					return true;
				} else {
					printf("Invalid NC approval...\n");
					return false;
				}
			} else printf("Runtime indication discarded (opcode: %d):\n", recent_runtime_indication);
		}
	}



	disconnect_reason yitran_rs::decode_disconnected_from_nc_indication() {
		switch ( recently_received_packet.payload[0] ) {
			case 0x00: return parent_unstable;
			case 0x01: return nvr_nack;
			case 0x02: return infinity;
			case 0x03: return init;
			case 0x04: return cant_start_timer;
			case 0x05: return nvr_refused;
			case 0x06: return nvr_enq;
			case 0x07: return invalid_node_id;
			case 0x08: return disconnected_by_application_request;
			default: return unknown_reason;
		}
	}



	void yitran_rs::decode_connected_to_nc_indication(yitran_node_descriptor& nd) {
		nd.parent_logical_id = (recently_received_packet.payload[1] << 8) | recently_received_packet.payload[0];
	}



	bool yitran_rs::wait_for_connected_to_nc_indication(unsigned timeout_ms) {
		get_packet(timeout_ms);
		if ( recently_received_packet.command_start == 0xca && recently_received_packet.opcode == 0xba ) return true;
		else {
			printf("wait_for_connected_to_nc_indication: Packet discarded\n");
			return false;
		}
	}



	bool yitran_rs::acknowledged_set_rs_operation_mode(unsigned timeout_ms) {
		set_rs_operation_mode();
		get_packet(timeout_ms);
		if ( recently_received_packet.payload[0] == 0x00 ) return false;
		else return true;
	}



	bool yitran_rs::verified_set_rs_operation_mode(unsigned timeout_ms) {
		if ( !acknowledged_set_rs_operation_mode(timeout_ms) ) return false;
		if ( get_operation_mode(timeout_ms) ) return false;
		else return true;
	}

}
//-------------------------------------------------------------------------------------------------
